c--------------------------------------------------------------
c     alphas/(4*pi) in terms of Lambda_QCD at NNLL
c--------------------------------------------------------------
c     based on mstw08/QCD-Pegasus (see alphas.f)

      function aspi(Q)
      implicit none
      real*8 aspi,Q,x,xlog
      integer nf
      real*8 b0,b1,b2,b1b0,b2b0,alphas
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      integer ord
      common /iorder/ord

      if(Q.lt.mc) then
        nf=3
        lambda=lambda3
      elseif(Q.ge.mc.and.Q.le.mb) then
        nf=4
        lambda=lambda4
      elseif(Q.gt.mb) then
        nf=5
        lambda=lambda5
      endif


c      b0 = 11d0-2d0*nf/3d0
c      b1 = 102d0-38d0*nf/3d0
c      b2 = 1428.5d0-(5033d0*nf)/18d0+(325d0*nf**2d0)/54d0
c      b1b0 = b1/(b0**2d0)
c      b2b0 = b2/(b0*b0*b0)

c      x = dlog(Q*Q/lambda/lambda)
c      xlog = dlog(x)
c      aspi = 1d0/b0/x - b1b0/b0*(xlog/x/x)
c     >          + b1b0*b1b0/b0*(xlog*xlog-xlog-1d0)/(x*x*x)
c     >          + b2b0/b0/(x*x*x)

c     calling the subroutine initialized in the main file
c     alphas(mu_r) returns the coupling constant (see alphas.f)
      aspi=alphas(Q)/(4*pi)

      return
      end

c--------------------------------------------------------------
c     Integrand to solve analytically at LL:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B ]
c     A = cusp. We take A0
c     B = non-cusp. At LL it is 0.
c--------------------------------------------------------------
      function funcspLL(xq,xup,xuplog,xlow,xlowlog,nf)
      implicit none
      real*8 funcspLL,xq,xup,xuplog,xlow,xlowlog
      integer nf
      real*8 zeta3,b0,G0
      real*8 CG0
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      G0 = 4d0*CA

      b0 = 11d0-2d0*nf/3d0

      CG0 = G0/(2d0*b0)*(-xup+xq*xuplog)
     >      -G0/(2d0*b0)*(-xlow+xq*xlowlog)

      funcspLL = CG0

      return
      end


c--------------------------------------------------------------
c     Integrand to solve analytically at NLL:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B ]
c     A = cusp. We take A0 and A1.
c     B = non-cusp. We take B0.
c--------------------------------------------------------------
      function funcspNLL(xq,xup,xuplog,xlow,xlowlog,nf)
      implicit none
      real*8 funcspNLL,xq,xup,xuplog,xlow,xlowlog
      integer nf
      real*8 zeta3,b0,b1,b2,b1b0,b1b02,G0,G1,gnc0
      real*8 CG0,CG1,Cgnc0
      real*8 xup2,xup3
      real*8 xlow2,xlow3
      real*8 xuplog2,xlowlog2
      data zeta3/1.20205690316d0/
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      b0 = 11d0-2d0*nf/3d0

      G0 = 4d0*CA
      G1 = 4d0*CA*( (67d0/9d0 - pi*pi/3d0)*CA - 10d0/9d0*nf )
      gnc0 = -2d0*b0

      b1 = 102d0-38d0*nf/3d0

      b1b0  = b1/(b0**2d0)
      b1b02 = b1b0*b1b0

      xup2=xup*xup
      xup3=xup2*xup

      xuplog2=xuplog*xuplog

      xlow2=xlow*xlow
      xlow3=xlow2*xlow

      xlowlog2=xlowlog*xlowlog

      Cgnc0 = gnc0/(2d0*b0)*(xuplog-b1b0*((-1d0-xuplog)/xup))
     >      - gnc0/(2d0*b0)*(xlowlog-b1b0*((-1d0-xlowlog)/xlow))

      CG0 = G0/(2d0*b0)*(-xup+xq*xuplog-b1b0*(-xq/xup-xq*xuplog/xup
     >      -xuplog*xuplog/2d0))
     >      -G0/(2d0*b0)*(-xlow+xq*xlowlog-b1b0*(-xq/xlow
     >      -xq*xlowlog/xlow-xlowlog*xlowlog/2d0))

      CG1 = G1/(2d0*b0*b0)*(-xq/xup-xuplog
     >      +b1b02*(1d0/4d0/xup2-2d0*xq/27d0/xup3+xuplog/2d0/xup2
     >      -2d0*xq*xuplog/9d0/xup3+xuplog2/2d0/xup2
     >      -xq*xuplog2/3d0/xup3)
     >      -2d0*b1b0*(1d0/xup-xq/4d0/xup2+xuplog/xup
     >      -xq*xuplog/2d0/xup2))
     >      -G1/(2d0*b0*b0)*(-xq/xlow-xlowlog
     >      +b1b02*(1d0/4d0/xlow2-2d0*xq/27d0/xlow3+xlowlog/2d0/xlow2
     >      -2d0*xq*xlowlog/9d0/xlow3+xlowlog2/2d0/xlow2
     >      -xq*xlowlog2/3d0/xlow3)
     >      -2d0*b1b0*(1d0/xlow-xq/4d0/xlow2+xlowlog/xlow
     >      -xq*xlowlog/2d0/xlow2))

      funcspNLL = Cgnc0 + CG0 + CG1

      return
      end


c--------------------------------------------------------------
c     Integrand to solve analytically at NNLL:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B ]
c     A = cusp. We take A0, A1 and A2.
c     B = non-cusp. We take B0 and B1.
c--------------------------------------------------------------
      function funcspNNLL(xq,xup,xuplog,xlow,xlowlog,nf)
      implicit none
      real*8 funcspNNLL,xq,xup,xuplog,xlow,xlowlog
      integer nf
      real*8 zeta3,b0,b1,b2,b1b0,b2b0,G0,G1,G2,gnc0,gnc1
      real*8 CG0,CG1,CG2,Cgnc0,Cgnc1
      real*8 b1b02,b1b03,b1b04,b1b05,b1b06,b2b02,b2b03
      real*8 xup2,xup3,xup4,xup5,xup6,xup7,xup8
      real*8 xlow2,xlow3,xlow4,xlow5,xlow6,xlow7,xlow8
      real*8 xuplog2,xuplog3,xuplog4,xuplog5,xuplog6,xuplog7
      real*8 xlowlog2,xlowlog3,xlowlog4,xlowlog5,xlowlog6,xlowlog7
      data zeta3/1.20205690316d0/
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      b0 = 11d0-2d0*nf/3d0
      b1 = 102d0-38d0*nf/3d0
      b2 = 1428.5d0-(5033d0*nf)/18d0+(325d0*nf**2d0)/54d0

      G0 = 4d0*CA
      G1 = 4d0*CA*( (67d0/9d0 - pi*pi/3d0)*CA - 10d0/9d0*nf )
      G2 = 4d0*CA*( -4d0/27d0*nf*nf + 3d0/2d0*nf*(-418d0/27d0
     >      + 40d0*pi*pi/27d0-56d0*zeta3/3d0) + 9d0*(245d0/6d0
     >      - 134d0*pi*pi/27d0 + 11d0*pi*pi*pi*pi/45d0
     >      + 22d0*zeta3/3d0) + 2d0/3d0*nf*(-55d0/3d0+16d0*zeta3) )

      gnc0 = -2d0*b0
      gnc1 = 2d0*CA*CA*(-692d0/27d0+11d0*pi*pi/18d0+2d0*zeta3)
     >      + CA*nf*(256d0/27d0-2d0*pi*pi/9d0) + 4d0*CF*nf

      b1b0  = b1/(b0**2d0)
      b1b02 = b1b0*b1b0
      b1b03 = b1b02*b1b0
      b1b04 = b1b03*b1b0
      b1b05 = b1b04*b1b0
      b1b06 = b1b05*b1b0

      b2b0  = b2/(b0*b0*b0)
      b2b02 = b2b0*b2b0
      b2b03 = b2b02*b2b0

      xup2=xup*xup
      xup3=xup2*xup
      xup4=xup3*xup
      xup5=xup4*xup
      xup6=xup5*xup
      xup7=xup6*xup
      xup8=xup7*xup

      xuplog2=xuplog*xuplog
      xuplog3=xuplog2*xuplog
      xuplog4=xuplog3*xuplog
      xuplog5=xuplog4*xuplog
      xuplog6=xuplog5*xuplog

      xlow2=xlow*xlow
      xlow3=xlow2*xlow
      xlow4=xlow3*xlow
      xlow5=xlow4*xlow
      xlow6=xlow5*xlow
      xlow7=xlow6*xlow
      xlow8=xlow7*xlow

      xlowlog2=xlowlog*xlowlog
      xlowlog3=xlowlog2*xlowlog
      xlowlog4=xlowlog3*xlowlog
      xlowlog5=xlowlog4*xlowlog
      xlowlog6=xlowlog5*xlowlog

      Cgnc0 = gnc0/(2d0*b0)*(xuplog-b1b0*((-1d0-xuplog)/xup)
     >      + b1b0*b1b0*((1d0-xuplog*xuplog)/(2d0*xup*xup))
     >      - b2b0/(2d0*xup*xup))
     >      - gnc0/(2d0*b0)*(xlowlog-b1b0*((-1d0-xlowlog)/xlow)
     >      + b1b0*b1b0*((1d0-xlowlog*xlowlog)/(2d0*xlow*xlow))
     >      - b2b0/(2d0*xlow*xlow))

      Cgnc1 = gnc1/(2d0*b0*b0)*(-1d0/xup+b1b02*(-2d0/27d0/xup3
     >      -xuplog2/3d0/xup3-2d0*xuplog/9d0/xup3)
     >      +b1b04*(-789d0/3125d0/xup5-xuplog4/5d0/xup5
     >      +6d0*xuplog3/25d0/xup5+43d0*xuplog2/125d0/xup5
     >      -164d0*xuplog/625d0/xup5)
     >      +b2b02*(-1d0/5d0/xup5) - 2d0*b1b0*(-1d0/4d0/xup2
     >      -xuplog/2d0/xup2)
     >      +2d0*b1b02*(10d0/27/xup3-xuplog2/3d0/xup3
     >      +xuplog/9d0/xup3) + 2d0*b2b0*(-1d0/xup3)
     >      -2d0*b1b03*(9d0/128d0/xup4-xuplog3/4d0/xup4
     >      +xuplog2/16d0/xup4+9d0*xuplog/32d0/xup4)
     >      -b1b0*b2b0*(-1d0/16d0/xup4-xuplog/4d0/xup4)
     >      +2d0*b1b02*b2b0*(28d0/125d0/xup5-xuplog2/5d0/xup5
     >      +3d0*xuplog/25d0/xup5))
     >      -gnc1/(2d0*b0*b0)*(-1d0/xlow+b1b02*(-2d0/27d0/xlow3
     >      -xlowlog2/3d0/xlow3-2d0*xlowlog/9d0/xlow3)
     >      +b1b04*(-789d0/3125d0/xlow5-xlowlog4/5d0/xlow5
     >      +6d0*xlowlog3/25d0/xlow5+43d0*xlowlog2/125d0/xlow5
     >      -164d0*xlowlog/625d0/xlow5)
     >      +b2b02*(-1d0/5d0/xlow5) - 2d0*b1b0*(-1d0/4d0/xlow2
     >      -xlowlog/2d0/xlow2)
     >      +2d0*b1b02*(10d0/27/xlow3-xlowlog2/3d0/xlow3
     >      +xlowlog/9d0/xlow3) + 2d0*b2b0*(-1d0/xlow3)
     >      -2d0*b1b03*(9d0/128d0/xlow4-xlowlog3/4d0/xlow4
     >      +xlowlog2/16d0/xlow4+9d0*xlowlog/32d0/xlow4)
     >      -b1b0*b2b0*(-1d0/16d0/xlow4-xlowlog/4d0/xlow4)
     >      +2d0*b1b02*b2b0*(28d0/125d0/xlow5-xlowlog2/5d0/xlow5
     >      +3d0*xlowlog/25d0/xlow5))

      CG0 = G0/(2d0*b0)*(-xup+xq*xuplog-b1b0*(-xq/xup-xq*xuplog/xup
     >      -xuplog*xuplog/2d0)+b1b0*b1b0*(((1d0+xuplog)
     >      *(xq+(2d0*xup-xq)*xuplog))/(2d0*xup*xup))
     >      +b2b0*(1d0/xup-xq/(2d0*xup*xup)))
     >      -G0/(2d0*b0)*(-xlow+xq*xlowlog-b1b0*(-xq/xlow
     >      -xq*xlowlog/xlow-xlowlog*xlowlog/2d0)+b1b0*b1b0
     >      *(((1d0+xlowlog)*(xq+(2d0*xlow-xq)*xlowlog))
     >      /(2d0*xlow*xlow))+b2b0*(1d0/xlow-xq/(2d0*xlow*xlow)))

      CG1 = G1/(2d0*b0*b0)*(-xq/xup-xuplog
     >      +b1b02*(1d0/4d0/xup2-2d0*xq/27d0/xup3+xuplog/2d0/xup2
     >      -2d0*xq*xuplog/9d0/xup3+xuplog2/2d0/xup2
     >      -xq*xuplog2/3d0/xup3)
     >      +b1b04/(400000d0*xup5)*(20000d0*(5d0*xup-4d0*xq)*xuplog4
     >      -4000d0*(25d0*xup-24d0*xq)*xuplog3
     >      -200d0*(875d0*xup-688d0*xq)*xuplog2
     >      +20d0*(5625d0*xup-5248d0*xq)*xuplog+128125d0*xup
     >      -100992*xq)
     >      +b2b03*(1d0/4d0/xup4-xq/5d0/xup5)
     >      -2d0*b1b0*(1d0/xup-xq/4d0/xup2+xuplog/xup
     >      -xq*xuplog/2d0/xup2)
     >      +2d0*b1b02*(9d0*(3d0*xup-2d0*xq)*xuplog2+6d0*xq*xuplog
     >      -27d0*xup+20d0*xq)/(54d0*xup3)
     >      +2d0*b2b0*(1d0/2d0/xup2-xq/3d0/xup3)
     >      -2d0*b1b03*(96d0*(4d0*xup-3d0*xq)*xuplog3
     >      +72d0*xq*xuplog2+(324d0*xq-384d0*xup)*xuplog
     >      -128d0*xup+81d0*xq)/(1152d0*xup4)
     >      -2d0*b1b0*b2b0*(-xq/16d0/xup4-xq*xuplog/4d0/xup4
     >      +1d0/9d0/xup3+xuplog/3d0/xup3)
     >      +2d0*b1b02*b2b0*(200d0*(5d0*xup-4d0*xq)*xuplog2
     >      +(480d0*xq-500d0*xup)*xuplog-1125d0*xup+896d0*xq)
     >      /(4000d0*xup5))
     >      -G1/(2d0*b0*b0)*(-xq/xlow-xlowlog
     >      +b1b02*(1d0/4d0/xlow2-2d0*xq/27d0/xlow3+xlowlog/2d0/xlow2
     >      -2d0*xq*xlowlog/9d0/xlow3+xlowlog2/2d0/xlow2
     >      -xq*xlowlog2/3d0/xlow3)
     >      +b1b04/(400000d0*xlow5)*(20000d0*(5d0*xlow-4d0*xq)*xlowlog4
     >      -4000d0*(25d0*xlow-24d0*xq)*xlowlog3
     >      -200d0*(875d0*xlow-688d0*xq)*xlowlog2
     >      +20d0*(5625d0*xlow-5248d0*xq)*xlowlog+128125d0*xlow
     >      -100992*xq)
     >      +b2b03*(1d0/4d0/xlow4-xq/5d0/xlow5)
     >      -2d0*b1b0*(1d0/xlow-xq/4d0/xlow2+xlowlog/xlow
     >      -xq*xlowlog/2d0/xlow2)
     >      +2d0*b1b02*(9d0*(3d0*xlow-2d0*xq)*xlowlog2+6d0*xq*xlowlog
     >      -27d0*xlow+20d0*xq)/(54d0*xlow3)
     >      +2d0*b2b0*(1d0/2d0/xlow2-xq/3d0/xlow3)
     >      -2d0*b1b03*(96d0*(4d0*xlow-3d0*xq)*xlowlog3
     >      +72d0*xq*xlowlog2+(324d0*xq-384d0*xlow)*xlowlog
     >      -128d0*xlow+81d0*xq)/(1152d0*xlow4)
     >      -2d0*b1b0*b2b0*(-xq/16d0/xlow4-xq*xlowlog/4d0/xlow4
     >      +1d0/9d0/xlow3+xlowlog/3d0/xlow3)
     >      +2d0*b1b02*b2b0*(200d0*(5d0*xlow-4d0*xq)*xlowlog2
     >      +(480d0*xq-500d0*xlow)*xlowlog-1125d0*xlow+896d0*xq)
     >      /(4000d0*xlow5))

      CG2 = G2/(2d0*b0*b0*b0)*(
     >      -b1b03*(3d0/(128d0*xup4)-(6d0*xq)/(625d0*xup5)
     >      +(3d0*xuplog)/(32d0*xup4)-(6d0*xq*xuplog)/(125d0*xup5)
     >      +(3d0*xuplog2)/(16d0*xup4)-(3d0*xq*xuplog2)/(25d0*xup5)
     >      +xuplog3/(4d0*xup4)-(xq*xuplog3)/(5d0*xup5))
     >      +b1b06*(-159580d0/(823543d0*xup7)
     >      +(21703d0*xq)/(131072d0*xup8)
     >      -(41931d0*xuplog)/(117649d0*xup7)
     >      +(5319d0*xq*xuplog)/(16384d0*xup8)
     >      +(4245d0*xuplog2)/(16807d0*xup7)
     >      -(825d0*xq*xuplog2)/(4096d0*xup8)
     >      +(1415d0*xuplog3)/(2401d0*xup7)
     >      -(275d0*xq*xuplog3)/(512d0*xup8)
     >      -(75d0*xuplog4)/(343d0*xup7)
     >      +(45d0*xq*xuplog4)/(256d0*xup8)
     >      -(15d0*xuplog5)/(49d0*xup7)
     >      +(9d0*xq*xuplog5)/(32d0*xup8)+xuplog6/(7d0*xup7)
     >      -(xq*xuplog6)/(8d0*xup8))
     >      +b2b03*(1d0/(7d0*xup7)-xq/(8d0*xup8))
     >      +1d0/xup-xq/(2d0*xup2)
     >      -3d0*b1b0*(1d0/(4d0*xup2)-xq/(9d0*xup3)+xuplog/(2d0*xup2)
     >      -(xq*xuplog)/(3d0*xup3))
     >      +3d0*b1b02*(2d0/(27d0*xup3)-xq/(32d0*xup4)
     >      +(2d0*xuplog)/(9d0*xup3)-(xq*xuplog)/(8d0*xup4)
     >      +xuplog2/(3d0*xup3)-(xq*xuplog2)/(4d0*xup4))
     >      +3d0*b1b02*(-10d0/(27d0*xup3)+(9d0*xq)/(32d0*xup4)
     >      -xuplog/(9d0*xup3)+(xq*xuplog)/(8d0*xup4)
     >      +xuplog2/(3d0*xup3)-(xq*xuplog2)/(4d0*xup4))
     >      -6d0*b1b03*(-9d0/(128d0*xup4)+(29d0*xq)/(625d0*xup5)
     >      -(9d0*xuplog)/(32d0*xup4)+(29d0*xq*xuplog)/(125d0*xup5)
     >      -xuplog2/(16d0*xup4)+(2d0*xq*xuplog2)/(25d0*xup5)
     >      +xuplog3/(4d0*xup4)-(xq*xuplog3)/(5d0*xup5))
     >      +3d0*b1b04*(-56d0/(3125d0*xup5)+(7d0*xq)/(648d0*xup6)
     >      -(56d0*xuplog)/(625d0*xup5)+(7d0*xq*xuplog)/(108d0*xup6)
     >      -(28d0*xuplog2)/(125d0*xup5)
     >      +(7d0*xq*xuplog2)/(36d0*xup6)-xuplog3/(25d0*xup5)
     >      +(xq*xuplog3)/(18d0*xup6)+xuplog4/(5d0*xup5)
     >      -(xq*xuplog4)/(6d0*xup6))
     >      +3d0*b1b04*(789d0/(3125d0*xup5)-(67d0*xq)/(324d0*xup6)
     >      +(164d0*xuplog)/(625d0*xup5)-(13d0*xq*xuplog)/(54d0*xup6)
     >      -(43d0*xuplog2)/(125d0*xup5)+(5d0*xq*xuplog2)/(18d0*xup6)
     >      -(6d0*xuplog3)/(25d0*xup5)+(2d0*xq*xuplog3)/(9d0*xup6)
     >      +xuplog4/(5d0*xup5)-(xq*xuplog4)/(6d0*xup6))
     >      -3d0*b1b05*(37d0/(972d0*xup6)-(3263d0*xq)/(117649d0*xup7)
     >      +(37d0*xuplog)/(162d0*xup6)
     >      -(3263d0*xq*xuplog)/(16807d0*xup7)
     >      +(5d0*xuplog2)/(27d0*xup6)
     >      -(431d0*xq*xuplog2)/(2401d0*xup7)
     >      -(8d0*xuplog3)/(27d0*xup6)
     >      +(85d0*xq*xuplog3)/(343d0*xup7)
     >      -(7d0*xuplog4)/(36d0*xup6)+(9d0*xq*xuplog4)/(49d0*xup7)
     >      +xuplog5/(6d0*xup6)-(xq*xuplog5)/(7d0*xup7))
     >      +3d0*b2b0*(1d0/(3d0*xup3)-xq/(4d0*xup4))
     >      -6d0*b1b0*b2b0*(1d0/(16d0*xup4)-xq/(25d0*xup5)
     >      +xuplog/(4d0*xup4)-(xq*xuplog)/(5d0*xup5))
     >      +3d0*b1b02*b2b0*(2d0/(125d0*xup5)-xq/(108d0*xup6)
     >      +(2d0*xuplog)/(25d0*xup5)-(xq*xuplog)/(18d0*xup6)
     >      +xuplog2/(5d0*xup5)-(xq*xuplog2)/(6d0*xup6))
     >      +6d0*b1b02*b2b0*(-28d0/(125d0*xup5)+(5d0*xq)/(27d0*xup6)
     >      -(3d0*xuplog)/(25d0*xup5)+(xq*xuplog)/(9d0*xup6)
     >      +xuplog2/(5d0*xup5)-(xq*xuplog2)/(6d0*xup6))
     >      -6d0*b1b03*b2b0*(-7d0/(216d0*xup6)
     >      +(57d0*xq)/(2401d0*xup7)
     >      -(7d0*xuplog)/(36d0*xup6)+(57d0*xq*xuplog)/(343d0*xup7)
     >      -xuplog2/(12d0*xup6)+(4d0*xq*xuplog2)/(49d0*xup7)
     >      +xuplog3/(6d0*xup6)-(xq*xuplog3)/(7d0*xup7))
     >      +3d0*b1b04*b2b0*(2929d0/(16807d0*xup7)
     >      -(615d0*xq)/(4096d0*xup8)
     >      +(528d0*xuplog)/(2401d0*xup7)
     >      -(103d0*xq*xuplog)/(512d0*xup8)
     >      -(79d0*xuplog2)/(343d0*xup7)
     >      +(25d0*xq*xuplog2)/(128d0*xup8)
     >      -(10d0*xuplog3)/(49d0*xup7)+(3d0*xq*xuplog3)/(16d0*xup8)
     >      +xuplog4/(7d0*xup7)-(xq*xuplog4)/(8d0*xup8))
     >      +3d0*b2b02*(1d0/(5d0*xup5)-xq/(6d0*xup6))
     >      -3d0*b1b0*b2b02*(1d0/(36d0*xup6)-xq/(49d0*xup7)
     >      +xuplog/(6d0*xup6)-(xq*xuplog)/(7d0*xup7))
     >      +3d0*b1b02*b2b02*(-54d0/(343d0*xup7)
     >      +(35d0*xq)/(256d0*xup8)
     >      -(5d0*xuplog)/(49d0*xup7)+(3d0*xq*xuplog)/(32d0*xup8)
     >      +xuplog2/(7d0*xup7)-(xq*xuplog2)/(8d0*xup8)))
     >      -G2/(2d0*b0*b0*b0)*(
     >      -b1b03*(3d0/(128d0*xlow4)-(6d0*xq)/(625d0*xlow5)
     >      +(3d0*xlowlog)/(32d0*xlow4)
     >      -(6d0*xq*xlowlog)/(125d0*xlow5)
     >      +(3d0*xlowlog2)/(16d0*xlow4)
     >      -(3d0*xq*xlowlog2)/(25d0*xlow5)
     >      +xlowlog3/(4d0*xlow4)-(xq*xlowlog3)/(5d0*xlow5))
     >      +b1b06*(-159580d0/(823543d0*xlow7)
     >      +(21703d0*xq)/(131072d0*xlow8)
     >      -(41931d0*xlowlog)/(117649d0*xlow7)
     >      +(5319d0*xq*xlowlog)/(16384d0*xlow8)
     >      +(4245d0*xlowlog2)/(16807d0*xlow7)
     >      -(825d0*xq*xlowlog2)/(4096d0*xlow8)
     >      +(1415d0*xlowlog3)/(2401d0*xlow7)
     >      -(275d0*xq*xlowlog3)/(512d0*xlow8)
     >      -(75d0*xlowlog4)/(343d0*xlow7)
     >      +(45d0*xq*xlowlog4)/(256d0*xlow8)
     >      -(15d0*xlowlog5)/(49d0*xlow7)
     >      +(9d0*xq*xlowlog5)/(32d0*xlow8)+xlowlog6/(7d0*xlow7)
     >      -(xq*xlowlog6)/(8d0*xlow8))
     >      +b2b03*(1d0/(7d0*xlow7)-xq/(8d0*xlow8))
     >      +1d0/xlow-xq/(2d0*xlow2)
     >      -3d0*b1b0*(1d0/(4d0*xlow2)
     >      -xq/(9d0*xlow3)+xlowlog/(2d0*xlow2)
     >      -(xq*xlowlog)/(3d0*xlow3))
     >      +3d0*b1b02*(2d0/(27d0*xlow3)-xq/(32d0*xlow4)
     >      +(2d0*xlowlog)/(9d0*xlow3)-(xq*xlowlog)/(8d0*xlow4)
     >      +xlowlog2/(3d0*xlow3)-(xq*xlowlog2)/(4d0*xlow4))
     >      +3d0*b1b02*(-10d0/(27d0*xlow3)+(9d0*xq)/(32d0*xlow4)
     >      -xlowlog/(9d0*xlow3)+(xq*xlowlog)/(8d0*xlow4)
     >      +xlowlog2/(3d0*xlow3)-(xq*xlowlog2)/(4d0*xlow4))
     >      -6d0*b1b03*(-9d0/(128d0*xlow4)+(29d0*xq)/(625d0*xlow5)
     >      -(9d0*xlowlog)/(32d0*xlow4)
     >      +(29d0*xq*xlowlog)/(125d0*xlow5)
     >      -xlowlog2/(16d0*xlow4)+(2d0*xq*xlowlog2)/(25d0*xlow5)
     >      +xlowlog3/(4d0*xlow4)-(xq*xlowlog3)/(5d0*xlow5))
     >      +3d0*b1b04*(-56d0/(3125d0*xlow5)+(7d0*xq)/(648d0*xlow6)
     >      -(56d0*xlowlog)/(625d0*xlow5)
     >      +(7d0*xq*xlowlog)/(108d0*xlow6)
     >      -(28d0*xlowlog2)/(125d0*xlow5)
     >      +(7d0*xq*xlowlog2)/(36d0*xlow6)-xlowlog3/(25d0*xlow5)
     >      +(xq*xlowlog3)/(18d0*xlow6)+xlowlog4/(5d0*xlow5)
     >      -(xq*xlowlog4)/(6d0*xlow6))
     >      +3d0*b1b04*(789d0/(3125d0*xlow5)-(67d0*xq)/(324d0*xlow6)
     >      +(164d0*xlowlog)/(625d0*xlow5)
     >      -(13d0*xq*xlowlog)/(54d0*xlow6)
     >      -(43d0*xlowlog2)/(125d0*xlow5)
     >      +(5d0*xq*xlowlog2)/(18d0*xlow6)
     >      -(6d0*xlowlog3)/(25d0*xlow5)
     >      +(2d0*xq*xlowlog3)/(9d0*xlow6)
     >      +xlowlog4/(5d0*xlow5)-(xq*xlowlog4)/(6d0*xlow6))
     >      -3d0*b1b05*(37d0/(972d0*xlow6)
     >      -(3263d0*xq)/(117649d0*xlow7)
     >      +(37d0*xlowlog)/(162d0*xlow6)
     >      -(3263d0*xq*xlowlog)/(16807d0*xlow7)
     >      +(5d0*xlowlog2)/(27d0*xlow6)
     >      -(431d0*xq*xlowlog2)/(2401d0*xlow7)
     >      -(8d0*xlowlog3)/(27d0*xlow6)
     >      +(85d0*xq*xlowlog3)/(343d0*xlow7)
     >      -(7d0*xlowlog4)/(36d0*xlow6)
     >      +(9d0*xq*xlowlog4)/(49d0*xlow7)
     >      +xlowlog5/(6d0*xlow6)-(xq*xlowlog5)/(7d0*xlow7))
     >      +3d0*b2b0*(1d0/(3d0*xlow3)-xq/(4d0*xlow4))
     >      -6d0*b1b0*b2b0*(1d0/(16d0*xlow4)-xq/(25d0*xlow5)
     >      +xlowlog/(4d0*xlow4)-(xq*xlowlog)/(5d0*xlow5))
     >      +3d0*b1b02*b2b0*(2d0/(125d0*xlow5)-xq/(108d0*xlow6)
     >      +(2d0*xlowlog)/(25d0*xlow5)-(xq*xlowlog)/(18d0*xlow6)
     >      +xlowlog2/(5d0*xlow5)-(xq*xlowlog2)/(6d0*xlow6))
     >      +6d0*b1b02*b2b0*(-28d0/(125d0*xlow5)
     >      +(5d0*xq)/(27d0*xlow6)
     >      -(3d0*xlowlog)/(25d0*xlow5)+(xq*xlowlog)/(9d0*xlow6)
     >      +xlowlog2/(5d0*xlow5)-(xq*xlowlog2)/(6d0*xlow6))
     >      -6d0*b1b03*b2b0*(-7d0/(216d0*xlow6)
     >      +(57d0*xq)/(2401d0*xlow7)
     >      -(7d0*xlowlog)/(36d0*xlow6)
     >      +(57d0*xq*xlowlog)/(343d0*xlow7)
     >      -xlowlog2/(12d0*xlow6)+(4d0*xq*xlowlog2)/(49d0*xlow7)
     >      +xlowlog3/(6d0*xlow6)-(xq*xlowlog3)/(7d0*xlow7))
     >      +3d0*b1b04*b2b0*(2929d0/(16807d0*xlow7)
     >      -(615d0*xq)/(4096d0*xlow8)
     >      +(528d0*xlowlog)/(2401d0*xlow7)
     >      -(103d0*xq*xlowlog)/(512d0*xlow8)
     >      -(79d0*xlowlog2)/(343d0*xlow7)
     >      +(25d0*xq*xlowlog2)/(128d0*xlow8)
     >      -(10d0*xlowlog3)/(49d0*xlow7)
     >      +(3d0*xq*xlowlog3)/(16d0*xlow8)
     >      +xlowlog4/(7d0*xlow7)-(xq*xlowlog4)/(8d0*xlow8))
     >      +3d0*b2b02*(1d0/(5d0*xlow5)-xq/(6d0*xlow6))
     >      -3d0*b1b0*b2b02*(1d0/(36d0*xlow6)-xq/(49d0*xlow7)
     >      +xlowlog/(6d0*xlow6)-(xq*xlowlog)/(7d0*xlow7))
     >      +3d0*b1b02*b2b02*(-54d0/(343d0*xlow7)
     >       +(35d0*xq)/(256d0*xlow8)
     >      -(5d0*xlowlog)/(49d0*xlow7)
     >      +(3d0*xq*xlowlog)/(32d0*xlow8)
     >      +xlowlog2/(7d0*xlow7)-(xq*xlowlog2)/(8d0*xlow8)))

      funcspNNLL = Cgnc0 + Cgnc1 + CG0 + CG1 + CG2

      return
      end

c--------------------------------------------------------------
c     Integrand to solve analytically:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B  ]
c     ord=0,1,2 for LL,NLL,NNLL
c--------------------------------------------------------------
      function funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
      real*8 funcsp,xq,xup,xuplog,xlow,xlowlog
      real*8 funcspLL,funcspNLL,funcspNNLL
      integer ord
      common /iorder/ord
      external funcspLL,funcspNLL,funcspNNLL

      if(ord.eq.0) then
        funcsp=funcspLL(xq,xup,xuplog,xlow,xlowlog,nf)
      elseif(ord.eq.1) then
        funcsp=funcspNLL(xq,xup,xuplog,xlow,xlowlog,nf)
      elseif(ord.eq.2) then
        funcsp=funcspNNLL(xq,xup,xuplog,xlow,xlowlog,nf)
      endif

      return
      end

c--------------------------------------------------------------
c     \exp[ - \int_{Qi}^{Qf} ( A \ln(Q^2/mu^2) + B ) ]
c     Inputs: Qi,Qf
c     The integrand is taken consistently at LL,NLL,NNLL depending on
c     the value of the common variable ord: 0,1,2 for LL,NLL,NNLL
c     Output: Revo
c--------------------------------------------------------------
      subroutine expgammaF(Qi,Qf,Revo)
      implicit none
      real*8 Qi,Qf,Revo
      real*8 funcsp
      integer nf
      real*8 xq,xup,xlow,xuplog,xlowlog,sp1,sp2,sp3,sp,lambda
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      if(Qi.le.mc) then
        if(Qf.le.mc) then
            nf = 3  ! Integrate from Qi to Qf
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
        elseif(Qf.gt.mc.and.Qf.le.mb) then
            nf = 3  ! Integrate from Qi to mc
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0 * dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mc to Qf
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        elseif(Qf.gt.mb) then
            nf = 3  ! Integrate from Qi to mc
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mc to mb
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 5  ! Integrate from mb to Qf
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp3 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2 + sp3
        endif
      elseif(Qi.gt.mc.and.Qi.le.mb) then
        if(Qf.le.mc) then
            nf = 4  ! Integrate from Qi to mc
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 3  ! Integrate from mc to Qf
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        elseif(Qf.gt.mc.and.Qf.le.mb) then
            nf = 4  ! Integrate from Qi to Qf
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
        elseif(Qf.gt.mb) then
            nf = 4  ! Integrate from Qi to mb
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 5  ! Integrate from mb to Qf
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        endif
      elseif(Qi.gt.mb) then
        if(Qf.le.mc) then
            nf = 5  ! Integrate from Qi to mb
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mb to mc
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 3  ! Integrate from mc to Qf
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp3 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2 + sp3
        elseif(Qf.gt.mc.and.Qf.le.mb) then
            nf = 5  ! Integrate from Qi to mb
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mb to Qf
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        elseif(Qf.gt.mb) then
            nf = 5  ! Integrate from Qi to Qf
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp = funcsp(xq,xup,xuplog,xlow,xlowlog,nf)
        endif
      endif

      Revo = dexp( -sp )

      return
      end


c--------------------------------------------------------------
c     Resummed D function ala EIS
c--------------------------------------------------------------
      subroutine DR(b,mu,res)
      implicit none
      real*8 res,b,mu,pi2,cusp0,cusp1,cusp2,zeta3
      real*8 beta0,beta1,beta2,X,alf,alf2,aspi,d20
      integer nf
      integer ord
      common /iorder/ord
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      external aspi

      pi2=pi*pi
      zeta3 = 1.20205690316d0

      if(mu.le.mc) then
        nf=3
      elseif(mu.gt.mc.and.mu.le.mb) then
        nf=4
      elseif(mu.gt.mb) then
        nf=5
      endif

      beta0 = 11d0-2d0/3d0*nf
      beta1 = 102d0-38d0/3d0*nf
      beta2 = 2857d0/54d0*CA*CA*CA
     $       +(2d0*CF*CF-205d0/9d0*CF*CA-1415d0/27d0*CA*CA)*nf/2d0
     $       + (44d0/9d0*CF+158d0/27d0*CA)*nf*nf/4d0

      cusp0=4d0*CA
      cusp1=4d0*CA*((67d0/9d0-pi2/3d0)*CA-10d0/9d0*nf)
      cusp2=4d0*CA*(CA*CA*(245d0/6d0-134d0*pi2/27d0+11d0*pi2*pi2/45d0
     $       +22d0/3d0*zeta3)+CA*nf/2d0*(-418d0/27d0+40d0*pi2/27d0
     $       -56d0*zeta3/3)+CF*nf/2d0*(-55d0/3d0+16d0*zeta3)
     $       - 16d0/27d0*nf*nf/4d0)


      if(ord.eq.0) then         ! LL
        alf = aspi(mu)
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        res = -cusp0/2d0/beta0*dlog(1d0-X)
      elseif(ord.eq.1) then     ! NLL
        alf = aspi(mu)
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        res = -cusp0/2d0/beta0*dlog(1d0-X)+1d0/2d0*alf/(1d0-X)*(-beta1
     $       /beta0/beta0*cusp0*(X+dlog(1d0-X))+cusp1/beta0*X)
      elseif(ord.eq.2) then     ! NNLL
        alf = aspi(mu)
        alf2 = alf*alf
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        d20 = CA*CA*(404d0/27d0-14d0*zeta3)-112d0/27d0*CA*nf/2d0
        res = -cusp0/2d0/beta0*dlog(1d0-X)+1d0/2d0*alf/(1d0-X)*(-beta1
     $       /beta0/beta0*cusp0*(X+dlog(1d0-X))+cusp1/beta0*X)
     $       +1d0/2d0*alf2/(1d0-X)/(1d0-X)*(2d0*d20
     $       +cusp2/2d0/beta0*(X*(2d0-X))+beta1*cusp1/2d0/beta0/beta0*
     $       (X*(X-2d0)-2d0*dlog(1d0-X))+beta2*cusp0/2d0/beta0/beta0*X*X
     $       +beta1*beta1*cusp0/2d0/beta0/beta0/beta0*(dlog(1d0-X)*
     $       dlog(1d0-X)-X*X))
      endif

      return
      end


c--------------------------------------------------------------
c     D function at fixed order, for m=mu_b
c--------------------------------------------------------------
      subroutine DRfixed(b,mu,res)
      implicit none
      real*8 res,b,mu,pi2,G0,G1,G2,zeta3,sc,sc2,lperp
      real*8 b0,b1,b2,X,alf,alf2,aspi,d20
      integer nf
      integer ord
      common /iorder/ord
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /scala/sc,sc2
      external aspi

      pi2=pi*pi
      zeta3 = 1.20205690316d0

      if(mu.le.mc) then
        nf=3
      elseif(mu.gt.mc.and.mu.le.mb) then
        nf=4
      elseif(mu.gt.mb) then
        nf=5
      endif

      b0 = 11d0-2d0/3d0*nf
      b1 = 102d0-38d0/3d0*nf
      b2 = 2857d0/54d0*CA*CA*CA
     $       +(2d0*CF*CF-205d0/9d0*CF*CA-1415d0/27d0*CA*CA)*nf/2d0
     $       + (44d0/9d0*CF+158d0/27d0*CA)*nf*nf/4d0

      G0=4d0*CA
      G1=4d0*CA*((67d0/9d0-pi2/3d0)*CA-10d0/9d0*nf)
      G2=4d0*CA*(CA*CA*(245d0/6d0-134d0*pi2/27d0+11d0*pi2*pi2/45d0
     $       +22d0/3d0*zeta3)+CA*nf/2d0*(-418d0/27d0+40d0*pi2/27d0
     $       -56d0*zeta3/3)+CF*nf/2d0*(-55d0/3d0+16d0*zeta3)
     $       - 16d0/27d0*nf*nf/4d0)

      lperp = dlog(mu*mu*b*b/c02)

      if(ord.eq.0) then         ! LL
        res = 0d0
      elseif(ord.eq.1) then     ! NLL
        alf = aspi(mu)
        res = alf*(G0/2d0)*lperp
      elseif(ord.eq.2) then     ! NNLL
        alf = aspi(mu)
        alf2 = alf*alf
        d20 = CA*CA*(404d0/27d0-14d0*zeta3)-112d0/27d0*CA*nf/2d0
        res = alf*(G0/2d0)*lperp
     $        + alf2*(G0/4d0*b0*lperp*lperp + G1/2d0*lperp + d20)
      endif

      return
      end


c--------------------------------------------------------------
c     Evolution kernel ala EIS: Revo (accurate only if Qf>>Qi)
c     The cutoff is put at bc=bX(Qi), see 1208.1281,
c     which should be more than enough for Qf>>Qi
c     because the kernel should go to zero before that.
c--------------------------------------------------------------
      subroutine kernel(order,b,Qi,Qf,result)
      implicit none
      real*8 b,Qi,Qf,result
      integer order,nf
      real*8 aspi,res,resDR,bc,b0
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      integer ord
      common /iorder/ord
      external aspi

      ord = order

      if(Qi.le.mc) then
        nf=3
      elseif(Qi.gt.mc.and.Qi.le.mb) then
        nf=4
      elseif(Qi.gt.mb) then
        nf=5
      endif

      b0 = 11d0-2d0*nf/3d0
      bc = c0/Qi*dexp(2d0*pi/b0/(4d0*pi*aspi(Qi)))

      if(b.lt.bc) then
        call expgammaF(Qi,Qf,res)
        call DR(b,Qi,resDR)
        result = res*dexp(-2d0*dlog(Qf/Qi)*resDR)
      elseif(b.ge.bc) then
        result = 0.d0
      endif

      return
      end



c--------------------------------------------------------------
c     Resummed h_\gamma function
c--------------------------------------------------------------
      subroutine hgR(b,mu,res)
      implicit none
      real*8 res,b,mu,pi2,gnc0,gnc1,gnc2,zeta3,zeta5
      real*8 beta0,beta1,beta2,X,alf,alf2,aspi,d20
      integer nf
      integer ord
      common /iorder/ord
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      external aspi

      pi2=pi*pi
      zeta3 = 1.20205690316d0
      zeta5 = 1.03692775514d0

      if(mu.le.mc) then
        nf=3
      elseif(mu.gt.mc.and.mu.le.mb) then
        nf=4
      elseif(mu.gt.mb) then
        nf=5
      endif

      beta0 = 11d0-2d0/3d0*nf
      beta1 = 102d0-38d0/3d0*nf
      beta2 = 2857d0/54d0*CA*CA*CA
     $       +(2d0*CF*CF-205d0/9d0*CF*CA-1415d0/27d0*CA*CA)*nf/2d0
     $       + (44d0/9d0*CF+158d0/27d0*CA)*nf*nf/4d0

      gnc0 = -2d0*beta0
      gnc1 = 2d0*CA*CA*(-692d0/27d0+11d0*pi*pi/18d0+2d0*zeta3)
     >      + CA*nf*(256d0/27d0-2d0*pi*pi/9d0) + 4d0*CF*nf
      gnc2 = (CA*CA*nf*(84.26611796982168d0 - (2396d0*pi*pi)/243d0
     >      + (164d0*pi**4d0)/135d0 + (1424d0*zeta3)/27d0))/2d0
     >      + 2d0*CA**3d0*(-133.31412894375856d0 + (6109d0*pi*pi)/486d0
     >      - (319d0*pi**4d0)/270d0 + (40.666666666666664d0
     >      - (20d0*pi*pi)/9d0)*zeta3 - 16d0*zeta5)


      if(ord.eq.0) then             ! LL
        res=0d0
      elseif(ord.eq.1) then         ! NLL
        alf = aspi(mu)
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        res = -gnc0/2d0/beta0*dlog(1d0-X)
      elseif(ord.eq.2) then         ! NNLL
        alf = aspi(mu)
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        res = -gnc0/2d0/beta0*dlog(1d0-X)+1d0/2d0*alf/(1d0-X)*(-beta1
     $       /beta0/beta0*gnc0*(X+dlog(1d0-X))+gnc1/beta0*X)
      elseif(ord.eq.3) then         ! NNNLL
        alf = aspi(mu)
        alf2 = alf*alf
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        zeta3 = 1.20205690316d0
        d20 = CA*CA*(404d0/27d0-14d0*zeta3)-112d0/27d0*CA*nf/2d0
        res = -gnc0/2d0/beta0*dlog(1d0-X)+1d0/2d0*alf/(1d0-X)*(-beta1
     $       /beta0/beta0*gnc0*(X+dlog(1d0-X))+gnc1/beta0*X)
     $       +1d0/2d0*alf2/(1d0-X)/(1d0-X)*(2d0*d20
     $       +gnc2/2d0/beta0*(X*(2d0-X))+beta1*gnc1/2d0/beta0/beta0*
     $       (X*(X-2d0)-2d0*dlog(1d0-X))+beta2*gnc0/2d0/beta0/beta0*X*X
     $       +beta1*beta1*gnc0/2d0/beta0/beta0/beta0*(dlog(1d0-X)*
     $       dlog(1d0-X)-X*X))
      endif


      return
      end



c--------------------------------------------------------------
c     h_\gamma function at fixed order, for m=mu_b
c--------------------------------------------------------------
      subroutine hgRfixed(b,mu,res)
      implicit none
      real*8 res,b,mu,pi2,gnc0,gnc1,gnc2,zeta3,zeta5,lperp
      real*8 b0,b1,b2,X,alf,alf2,aspi,d20,sc,sc2
      integer nf
      integer ord
      common /iorder/ord
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /scala/sc,sc2
      external aspi

      pi2=pi*pi
      zeta3 = 1.20205690316d0
      zeta5 = 1.03692775514d0

      if(mu.le.mc) then
        nf=3
      elseif(mu.gt.mc.and.mu.le.mb) then
        nf=4
      elseif(mu.gt.mb) then
        nf=5
      endif

      b0 = 11d0-2d0/3d0*nf
      b1 = 102d0-38d0/3d0*nf
      b2 = 2857d0/54d0*CA*CA*CA
     $       +(2d0*CF*CF-205d0/9d0*CF*CA-1415d0/27d0*CA*CA)*nf/2d0
     $       + (44d0/9d0*CF+158d0/27d0*CA)*nf*nf/4d0

      gnc0 = -2d0*b0
      gnc1 = 2d0*CA*CA*(-692d0/27d0+11d0*pi*pi/18d0+2d0*zeta3)
     >      + CA*nf*(256d0/27d0-2d0*pi*pi/9d0) + 4d0*CF*nf
      gnc2 = (CA*CA*nf*(84.26611796982168d0 - (2396d0*pi*pi)/243d0
     >      + (164d0*pi**4d0)/135d0 + (1424d0*zeta3)/27d0))/2d0
     >      + 2d0*CA**3d0*(-133.31412894375856d0 + (6109d0*pi*pi)/486d0
     >      - (319d0*pi**4d0)/270d0 + (40.666666666666664d0
     >      - (20d0*pi*pi)/9d0)*zeta3 - 16d0*zeta5)

      lperp = dlog(mu*mu*b*b/c02)


      if(ord.eq.0) then             ! LL
        res=0d0
      elseif(ord.eq.1) then         ! NLL
        alf = aspi(mu)
        res = alf*(gnc0/2d0)*lperp
      elseif(ord.eq.2) then         ! NNLL
        alf = aspi(mu)
        alf2 = alf*alf
        res = alf*(gnc0/2d0)*lperp
     $        + alf2*(gnc0/4d0*b0*lperp*lperp + gnc1/2d0*lperp)
      endif


      return
      end


c--------------------------------------------------------------
c     Resummed h_\Gamma function
c--------------------------------------------------------------
      subroutine hggR(b,mu,res)
      implicit none
      real*8 res,b,mu,pi2,cusp0,cusp1,cusp2,zeta3,zeta5
      real*8 beta0,beta1,beta2,X,alf,alf2,aspi,d20
      integer nf
      integer ord
      common /iorder/ord
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      external aspi

      pi2=pi*pi
      zeta3 = 1.20205690316d0
      zeta5 = 1.03692775514d0

      if(mu.le.mc) then
        nf=3
      elseif(mu.gt.mc.and.mu.le.mb) then
        nf=4
      elseif(mu.gt.mb) then
        nf=5
      endif

      beta0 = 11d0-2d0/3d0*nf
      beta1 = 102d0-38d0/3d0*nf
      beta2 = 2857d0/54d0*CA*CA*CA
     $       +(2d0*CF*CF-205d0/9d0*CF*CA-1415d0/27d0*CA*CA)*nf/2d0
     $       + (44d0/9d0*CF+158d0/27d0*CA)*nf*nf/4d0

      cusp0=4d0*CA
      cusp1=4d0*CA*((67d0/9d0-pi2/3d0)*CA-10d0/9d0*nf)
      cusp2=4d0*CA*(CA*CA*(245d0/6d0-134d0*pi2/27d0+11d0*pi2*pi2/45d0
     $       +22d0/3d0*zeta3)+CA*nf/2d0*(-418d0/27d0+40d0*pi2/27d0
     $       -56d0*zeta3/3)+CF*nf/2d0*(-55d0/3d0+16d0*zeta3)
     $       - 16d0/27d0*nf*nf/4d0)


      if(ord.eq.0) then         ! LL
        alf = aspi(mu)
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        res = cusp0*(X-(X-1d0)*dlog(1d0-X))/(2d0*alf*beta0*beta0)
      elseif(ord.eq.1) then     ! NLL
        alf = aspi(mu)
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        res = cusp0*(X-(X-1d0)*dlog(1d0-X))/(2d0*alf*beta0*beta0)
     $        +(beta1*cusp0*(2d0*X+(dlog(1d0-X))**2d0+2d0*dlog(1d0-X))
     $        -2d0*beta0*cusp1*(X+dlog(1d0-X)))
     $          /(4d0*beta0**3d0)
      elseif(ord.eq.2) then     ! NNLL
        alf = aspi(mu)
        alf2 = alf*alf
        X = alf*beta0*dlog(mu*mu*b*b/c02)
        res = cusp0*(X-(X-1d0)*dlog(1d0-X))/(2d0*alf*beta0*beta0)
     $        +(beta1*cusp0*(2d0*X+(dlog(1d0-X))**2d0+2d0*dlog(1d0-X))
     $        -2d0*beta0*cusp1*(X+dlog(1d0-X)))
     $          /(4d0*beta0**3d0)
     $       + alf/(4d0*beta0**4d0*(1d0-X))*(beta0*beta0*cusp2*X*X
     $       -beta0*(beta1*cusp1*(X*(X-2d0)+2d0*dlog(1d0-X))
     $       +beta0*cusp0*((X-2d0)*X+2d0*(X-1d0)*dlog(1d0-X)))
     $       +beta1*beta1*cusp0*(X+dlog(1d0-X))**2d0)
      endif


      return
      end


c--------------------------------------------------------------
c     h_\Gamma function at fixed order, for m=mu_b
c--------------------------------------------------------------
      subroutine hggRfixed(b,mu,res)
      implicit none
      real*8 res,b,mu,pi2,G0,G1,G2,zeta3,zeta5
      real*8 b0,b1,b2,X,alf,alf2,aspi,d20,sc,sc2,lperp
      integer nf
      integer ord
      common /iorder/ord
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /scala/sc,sc2
      external aspi

      pi2=pi*pi
      zeta3 = 1.20205690316d0
      zeta5 = 1.03692775514d0

      if(mu.le.mc) then
        nf=3
      elseif(mu.gt.mc.and.mu.le.mb) then
        nf=4
      elseif(mu.gt.mb) then
        nf=5
      endif

      b0 = 11d0-2d0/3d0*nf
      b1 = 102d0-38d0/3d0*nf
      b2 = 2857d0/54d0*CA*CA*CA
     $       +(2d0*CF*CF-205d0/9d0*CF*CA-1415d0/27d0*CA*CA)*nf/2d0
     $       + (44d0/9d0*CF+158d0/27d0*CA)*nf*nf/4d0

      G0=4d0*CA
      G1=4d0*CA*((67d0/9d0-pi2/3d0)*CA-10d0/9d0*nf)
      G2=4d0*CA*(CA*CA*(245d0/6d0-134d0*pi2/27d0+11d0*pi2*pi2/45d0
     $       +22d0/3d0*zeta3)+CA*nf/2d0*(-418d0/27d0+40d0*pi2/27d0
     $       -56d0*zeta3/3)+CF*nf/2d0*(-55d0/3d0+16d0*zeta3)
     $       - 16d0/27d0*nf*nf/4d0)

      lperp = dlog(mu*mu*b*b/c02)

      if(ord.eq.0) then         ! LL
        res = 0d0
      elseif(ord.eq.1) then     ! NLL
        alf = aspi(mu)
        res = alf*(G0/4d0)*lperp*lperp
      elseif(ord.eq.2) then     ! NNLL
        alf = aspi(mu)
        alf2 = alf*alf
        res = alf*(G0/4d0)*lperp*lperp
     $    + alf2/12d0*(G0*b0*lperp*lperp*lperp+3d0*G1*lperp*lperp)
      endif


      return
      end



c--------------------------------------------------------------
c     EVOLUTION OF THE HARD PART
c     Integrand to solve analytically at LL:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B ]
c     A = cusp. We take A0
c     B = non-cusp. At LL it is 0.
c--------------------------------------------------------------
      function funcspLLh(xq,xup,xuplog,xlow,xlowlog,nf)
      implicit none
      real*8 funcspLLh,xq,xup,xuplog,xlow,xlowlog
      integer nf
      real*8 zeta3,b0,G0
      real*8 CG0
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      G0 = 4d0*CA

      b0 = 11d0-2d0*nf/3d0

      CG0 = G0/(2d0*b0)*(-xup+xq*xuplog)
     >      -G0/(2d0*b0)*(-xlow+xq*xlowlog)

      funcspLLh = CG0

      return
      end


c--------------------------------------------------------------
c     EVOLUTION OF THE HARD PART
c     Integrand to solve analytically at NLL:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B ]
c     A = cusp. We take A0 and A1.
c     B = non-cusp. We take B0.
c--------------------------------------------------------------
      function funcspNLLh(xq,xup,xuplog,xlow,xlowlog,nf)
      implicit none
      real*8 funcspNLLh,xq,xup,xuplog,xlow,xlowlog
      integer nf
      real*8 zeta3,b0,b1,b2,b1b0,b1b02,G0,G1,gnc0
      real*8 CG0,CG1,Cgnc0
      real*8 xup2,xup3
      real*8 xlow2,xlow3
      real*8 xuplog2,xlowlog2
      data zeta3/1.20205690316d0/
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      b0 = 11d0-2d0*nf/3d0

      G0 = 4d0*CA
      G1 = 4d0*CA*( (67d0/9d0 - pi*pi/3d0)*CA - 10d0/9d0*nf )
      gnc0 = 0d0

      b1 = 102d0-38d0*nf/3d0

      b1b0  = b1/(b0**2d0)
      b1b02 = b1b0*b1b0

      xup2=xup*xup
      xup3=xup2*xup

      xuplog2=xuplog*xuplog

      xlow2=xlow*xlow
      xlow3=xlow2*xlow

      xlowlog2=xlowlog*xlowlog

      Cgnc0 = gnc0/(2d0*b0)*(xuplog-b1b0*((-1d0-xuplog)/xup))
     >      - gnc0/(2d0*b0)*(xlowlog-b1b0*((-1d0-xlowlog)/xlow))

      CG0 = G0/(2d0*b0)*(-xup+xq*xuplog-b1b0*(-xq/xup-xq*xuplog/xup
     >      -xuplog*xuplog/2d0))
     >      -G0/(2d0*b0)*(-xlow+xq*xlowlog-b1b0*(-xq/xlow
     >      -xq*xlowlog/xlow-xlowlog*xlowlog/2d0))

      CG1 = G1/(2d0*b0*b0)*(-xq/xup-xuplog
     >      +b1b02*(1d0/4d0/xup2-2d0*xq/27d0/xup3+xuplog/2d0/xup2
     >      -2d0*xq*xuplog/9d0/xup3+xuplog2/2d0/xup2
     >      -xq*xuplog2/3d0/xup3)
     >      -2d0*b1b0*(1d0/xup-xq/4d0/xup2+xuplog/xup
     >      -xq*xuplog/2d0/xup2))
     >      -G1/(2d0*b0*b0)*(-xq/xlow-xlowlog
     >      +b1b02*(1d0/4d0/xlow2-2d0*xq/27d0/xlow3+xlowlog/2d0/xlow2
     >      -2d0*xq*xlowlog/9d0/xlow3+xlowlog2/2d0/xlow2
     >      -xq*xlowlog2/3d0/xlow3)
     >      -2d0*b1b0*(1d0/xlow-xq/4d0/xlow2+xlowlog/xlow
     >      -xq*xlowlog/2d0/xlow2))

      funcspNLLh = Cgnc0 + CG0 + CG1

      return
      end


c--------------------------------------------------------------
c     EVOLUTION OF THE HARD PART
c     Integrand to solve analytically at NNLL:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B ]
c     A = cusp. We take A0, A1 and A2.
c     B = non-cusp. We take B0 and B1.
c--------------------------------------------------------------
      function funcspNNLLh(xq,xup,xuplog,xlow,xlowlog,nf)
      implicit none
      real*8 funcspNNLLh,xq,xup,xuplog,xlow,xlowlog
      integer nf
      real*8 zeta3,b0,b1,b2,b1b0,b2b0,G0,G1,G2,gnc0,gnc1
      real*8 CG0,CG1,CG2,Cgnc0,Cgnc1
      real*8 b1b02,b1b03,b1b04,b1b05,b1b06,b2b02,b2b03
      real*8 xup2,xup3,xup4,xup5,xup6,xup7,xup8
      real*8 xlow2,xlow3,xlow4,xlow5,xlow6,xlow7,xlow8
      real*8 xuplog2,xuplog3,xuplog4,xuplog5,xuplog6,xuplog7
      real*8 xlowlog2,xlowlog3,xlowlog4,xlowlog5,xlowlog6,xlowlog7
      data zeta3/1.20205690316d0/
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      b0 = 11d0-2d0*nf/3d0
      b1 = 102d0-38d0*nf/3d0
      b2 = 1428.5d0-(5033d0*nf)/18d0+(325d0*nf**2d0)/54d0

      G0 = 4d0*CA
      G1 = 4d0*CA*( (67d0/9d0 - pi*pi/3d0)*CA - 10d0/9d0*nf )
      G2 = 4d0*CA*( -4d0/27d0*nf*nf + 3d0/2d0*nf*(-418d0/27d0
     >      + 40d0*pi*pi/27d0-56d0*zeta3/3d0) + 9d0*(245d0/6d0
     >      - 134d0*pi*pi/27d0 + 11d0*pi*pi*pi*pi/45d0
     >      + 22d0*zeta3/3d0) + 2d0/3d0*nf*(-55d0/3d0+16d0*zeta3) )

      gnc0 = 0d0
      gnc1 = CA*CA*(-160d0/27d0+11d0*pi*pi/9d0+4d0*zeta3)
     >      + CA*nf*(-104d0/27d0-2d0*pi*pi/9d0) - 4d0*CF*nf

      b1b0  = b1/(b0**2d0)
      b1b02 = b1b0*b1b0
      b1b03 = b1b02*b1b0
      b1b04 = b1b03*b1b0
      b1b05 = b1b04*b1b0
      b1b06 = b1b05*b1b0

      b2b0  = b2/(b0*b0*b0)
      b2b02 = b2b0*b2b0
      b2b03 = b2b02*b2b0

      xup2=xup*xup
      xup3=xup2*xup
      xup4=xup3*xup
      xup5=xup4*xup
      xup6=xup5*xup
      xup7=xup6*xup
      xup8=xup7*xup

      xuplog2=xuplog*xuplog
      xuplog3=xuplog2*xuplog
      xuplog4=xuplog3*xuplog
      xuplog5=xuplog4*xuplog
      xuplog6=xuplog5*xuplog

      xlow2=xlow*xlow
      xlow3=xlow2*xlow
      xlow4=xlow3*xlow
      xlow5=xlow4*xlow
      xlow6=xlow5*xlow
      xlow7=xlow6*xlow
      xlow8=xlow7*xlow

      xlowlog2=xlowlog*xlowlog
      xlowlog3=xlowlog2*xlowlog
      xlowlog4=xlowlog3*xlowlog
      xlowlog5=xlowlog4*xlowlog
      xlowlog6=xlowlog5*xlowlog

      Cgnc0 = gnc0/(2d0*b0)*(xuplog-b1b0*((-1d0-xuplog)/xup)
     >      + b1b0*b1b0*((1d0-xuplog*xuplog)/(2d0*xup*xup))
     >      - b2b0/(2d0*xup*xup))
     >      - gnc0/(2d0*b0)*(xlowlog-b1b0*((-1d0-xlowlog)/xlow)
     >      + b1b0*b1b0*((1d0-xlowlog*xlowlog)/(2d0*xlow*xlow))
     >      - b2b0/(2d0*xlow*xlow))

      Cgnc1 = gnc1/(2d0*b0*b0)*(-1d0/xup+b1b02*(-2d0/27d0/xup3
     >      -xuplog2/3d0/xup3-2d0*xuplog/9d0/xup3)
     >      +b1b04*(-789d0/3125d0/xup5-xuplog4/5d0/xup5
     >      +6d0*xuplog3/25d0/xup5+43d0*xuplog2/125d0/xup5
     >      -164d0*xuplog/625d0/xup5)
     >      +b2b02*(-1d0/5d0/xup5) - 2d0*b1b0*(-1d0/4d0/xup2
     >      -xuplog/2d0/xup2)
     >      +2d0*b1b02*(10d0/27/xup3-xuplog2/3d0/xup3
     >      +xuplog/9d0/xup3) + 2d0*b2b0*(-1d0/xup3)
     >      -2d0*b1b03*(9d0/128d0/xup4-xuplog3/4d0/xup4
     >      +xuplog2/16d0/xup4+9d0*xuplog/32d0/xup4)
     >      -b1b0*b2b0*(-1d0/16d0/xup4-xuplog/4d0/xup4)
     >      +2d0*b1b02*b2b0*(28d0/125d0/xup5-xuplog2/5d0/xup5
     >      +3d0*xuplog/25d0/xup5))
     >      -gnc1/(2d0*b0*b0)*(-1d0/xlow+b1b02*(-2d0/27d0/xlow3
     >      -xlowlog2/3d0/xlow3-2d0*xlowlog/9d0/xlow3)
     >      +b1b04*(-789d0/3125d0/xlow5-xlowlog4/5d0/xlow5
     >      +6d0*xlowlog3/25d0/xlow5+43d0*xlowlog2/125d0/xlow5
     >      -164d0*xlowlog/625d0/xlow5)
     >      +b2b02*(-1d0/5d0/xlow5) - 2d0*b1b0*(-1d0/4d0/xlow2
     >      -xlowlog/2d0/xlow2)
     >      +2d0*b1b02*(10d0/27/xlow3-xlowlog2/3d0/xlow3
     >      +xlowlog/9d0/xlow3) + 2d0*b2b0*(-1d0/xlow3)
     >      -2d0*b1b03*(9d0/128d0/xlow4-xlowlog3/4d0/xlow4
     >      +xlowlog2/16d0/xlow4+9d0*xlowlog/32d0/xlow4)
     >      -b1b0*b2b0*(-1d0/16d0/xlow4-xlowlog/4d0/xlow4)
     >      +2d0*b1b02*b2b0*(28d0/125d0/xlow5-xlowlog2/5d0/xlow5
     >      +3d0*xlowlog/25d0/xlow5))

      CG0 = G0/(2d0*b0)*(-xup+xq*xuplog-b1b0*(-xq/xup-xq*xuplog/xup
     >      -xuplog*xuplog/2d0)+b1b0*b1b0*(((1d0+xuplog)
     >      *(xq+(2d0*xup-xq)*xuplog))/(2d0*xup*xup))
     >      +b2b0*(1d0/xup-xq/(2d0*xup*xup)))
     >      -G0/(2d0*b0)*(-xlow+xq*xlowlog-b1b0*(-xq/xlow
     >      -xq*xlowlog/xlow-xlowlog*xlowlog/2d0)+b1b0*b1b0
     >      *(((1d0+xlowlog)*(xq+(2d0*xlow-xq)*xlowlog))
     >      /(2d0*xlow*xlow))+b2b0*(1d0/xlow-xq/(2d0*xlow*xlow)))

      CG1 = G1/(2d0*b0*b0)*(-xq/xup-xuplog
     >      +b1b02*(1d0/4d0/xup2-2d0*xq/27d0/xup3+xuplog/2d0/xup2
     >      -2d0*xq*xuplog/9d0/xup3+xuplog2/2d0/xup2
     >      -xq*xuplog2/3d0/xup3)
     >      +b1b04/(400000d0*xup5)*(20000d0*(5d0*xup-4d0*xq)*xuplog4
     >      -4000d0*(25d0*xup-24d0*xq)*xuplog3
     >      -200d0*(875d0*xup-688d0*xq)*xuplog2
     >      +20d0*(5625d0*xup-5248d0*xq)*xuplog+128125d0*xup
     >      -100992*xq)
     >      +b2b03*(1d0/4d0/xup4-xq/5d0/xup5)
     >      -2d0*b1b0*(1d0/xup-xq/4d0/xup2+xuplog/xup
     >      -xq*xuplog/2d0/xup2)
     >      +2d0*b1b02*(9d0*(3d0*xup-2d0*xq)*xuplog2+6d0*xq*xuplog
     >      -27d0*xup+20d0*xq)/(54d0*xup3)
     >      +2d0*b2b0*(1d0/2d0/xup2-xq/3d0/xup3)
     >      -2d0*b1b03*(96d0*(4d0*xup-3d0*xq)*xuplog3
     >      +72d0*xq*xuplog2+(324d0*xq-384d0*xup)*xuplog
     >      -128d0*xup+81d0*xq)/(1152d0*xup4)
     >      -2d0*b1b0*b2b0*(-xq/16d0/xup4-xq*xuplog/4d0/xup4
     >      +1d0/9d0/xup3+xuplog/3d0/xup3)
     >      +2d0*b1b02*b2b0*(200d0*(5d0*xup-4d0*xq)*xuplog2
     >      +(480d0*xq-500d0*xup)*xuplog-1125d0*xup+896d0*xq)
     >      /(4000d0*xup5))
     >      -G1/(2d0*b0*b0)*(-xq/xlow-xlowlog
     >      +b1b02*(1d0/4d0/xlow2-2d0*xq/27d0/xlow3+xlowlog/2d0/xlow2
     >      -2d0*xq*xlowlog/9d0/xlow3+xlowlog2/2d0/xlow2
     >      -xq*xlowlog2/3d0/xlow3)
     >      +b1b04/(400000d0*xlow5)*(20000d0*(5d0*xlow-4d0*xq)*xlowlog4
     >      -4000d0*(25d0*xlow-24d0*xq)*xlowlog3
     >      -200d0*(875d0*xlow-688d0*xq)*xlowlog2
     >      +20d0*(5625d0*xlow-5248d0*xq)*xlowlog+128125d0*xlow
     >      -100992*xq)
     >      +b2b03*(1d0/4d0/xlow4-xq/5d0/xlow5)
     >      -2d0*b1b0*(1d0/xlow-xq/4d0/xlow2+xlowlog/xlow
     >      -xq*xlowlog/2d0/xlow2)
     >      +2d0*b1b02*(9d0*(3d0*xlow-2d0*xq)*xlowlog2+6d0*xq*xlowlog
     >      -27d0*xlow+20d0*xq)/(54d0*xlow3)
     >      +2d0*b2b0*(1d0/2d0/xlow2-xq/3d0/xlow3)
     >      -2d0*b1b03*(96d0*(4d0*xlow-3d0*xq)*xlowlog3
     >      +72d0*xq*xlowlog2+(324d0*xq-384d0*xlow)*xlowlog
     >      -128d0*xlow+81d0*xq)/(1152d0*xlow4)
     >      -2d0*b1b0*b2b0*(-xq/16d0/xlow4-xq*xlowlog/4d0/xlow4
     >      +1d0/9d0/xlow3+xlowlog/3d0/xlow3)
     >      +2d0*b1b02*b2b0*(200d0*(5d0*xlow-4d0*xq)*xlowlog2
     >      +(480d0*xq-500d0*xlow)*xlowlog-1125d0*xlow+896d0*xq)
     >      /(4000d0*xlow5))

      CG2 = G2/(2d0*b0*b0*b0)*(
     >      -b1b03*(3d0/(128d0*xup4)-(6d0*xq)/(625d0*xup5)
     >      +(3d0*xuplog)/(32d0*xup4)-(6d0*xq*xuplog)/(125d0*xup5)
     >      +(3d0*xuplog2)/(16d0*xup4)-(3d0*xq*xuplog2)/(25d0*xup5)
     >      +xuplog3/(4d0*xup4)-(xq*xuplog3)/(5d0*xup5))
     >      +b1b06*(-159580d0/(823543d0*xup7)
     >      +(21703d0*xq)/(131072d0*xup8)
     >      -(41931d0*xuplog)/(117649d0*xup7)
     >      +(5319d0*xq*xuplog)/(16384d0*xup8)
     >      +(4245d0*xuplog2)/(16807d0*xup7)
     >      -(825d0*xq*xuplog2)/(4096d0*xup8)
     >      +(1415d0*xuplog3)/(2401d0*xup7)
     >      -(275d0*xq*xuplog3)/(512d0*xup8)
     >      -(75d0*xuplog4)/(343d0*xup7)
     >      +(45d0*xq*xuplog4)/(256d0*xup8)
     >      -(15d0*xuplog5)/(49d0*xup7)
     >      +(9d0*xq*xuplog5)/(32d0*xup8)+xuplog6/(7d0*xup7)
     >      -(xq*xuplog6)/(8d0*xup8))
     >      +b2b03*(1d0/(7d0*xup7)-xq/(8d0*xup8))
     >      +1d0/xup-xq/(2d0*xup2)
     >      -3d0*b1b0*(1d0/(4d0*xup2)-xq/(9d0*xup3)+xuplog/(2d0*xup2)
     >      -(xq*xuplog)/(3d0*xup3))
     >      +3d0*b1b02*(2d0/(27d0*xup3)-xq/(32d0*xup4)
     >      +(2d0*xuplog)/(9d0*xup3)-(xq*xuplog)/(8d0*xup4)
     >      +xuplog2/(3d0*xup3)-(xq*xuplog2)/(4d0*xup4))
     >      +3d0*b1b02*(-10d0/(27d0*xup3)+(9d0*xq)/(32d0*xup4)
     >      -xuplog/(9d0*xup3)+(xq*xuplog)/(8d0*xup4)
     >      +xuplog2/(3d0*xup3)-(xq*xuplog2)/(4d0*xup4))
     >      -6d0*b1b03*(-9d0/(128d0*xup4)+(29d0*xq)/(625d0*xup5)
     >      -(9d0*xuplog)/(32d0*xup4)+(29d0*xq*xuplog)/(125d0*xup5)
     >      -xuplog2/(16d0*xup4)+(2d0*xq*xuplog2)/(25d0*xup5)
     >      +xuplog3/(4d0*xup4)-(xq*xuplog3)/(5d0*xup5))
     >      +3d0*b1b04*(-56d0/(3125d0*xup5)+(7d0*xq)/(648d0*xup6)
     >      -(56d0*xuplog)/(625d0*xup5)+(7d0*xq*xuplog)/(108d0*xup6)
     >      -(28d0*xuplog2)/(125d0*xup5)
     >      +(7d0*xq*xuplog2)/(36d0*xup6)-xuplog3/(25d0*xup5)
     >      +(xq*xuplog3)/(18d0*xup6)+xuplog4/(5d0*xup5)
     >      -(xq*xuplog4)/(6d0*xup6))
     >      +3d0*b1b04*(789d0/(3125d0*xup5)-(67d0*xq)/(324d0*xup6)
     >      +(164d0*xuplog)/(625d0*xup5)-(13d0*xq*xuplog)/(54d0*xup6)
     >      -(43d0*xuplog2)/(125d0*xup5)+(5d0*xq*xuplog2)/(18d0*xup6)
     >      -(6d0*xuplog3)/(25d0*xup5)+(2d0*xq*xuplog3)/(9d0*xup6)
     >      +xuplog4/(5d0*xup5)-(xq*xuplog4)/(6d0*xup6))
     >      -3d0*b1b05*(37d0/(972d0*xup6)-(3263d0*xq)/(117649d0*xup7)
     >      +(37d0*xuplog)/(162d0*xup6)
     >      -(3263d0*xq*xuplog)/(16807d0*xup7)
     >      +(5d0*xuplog2)/(27d0*xup6)
     >      -(431d0*xq*xuplog2)/(2401d0*xup7)
     >      -(8d0*xuplog3)/(27d0*xup6)
     >      +(85d0*xq*xuplog3)/(343d0*xup7)
     >      -(7d0*xuplog4)/(36d0*xup6)+(9d0*xq*xuplog4)/(49d0*xup7)
     >      +xuplog5/(6d0*xup6)-(xq*xuplog5)/(7d0*xup7))
     >      +3d0*b2b0*(1d0/(3d0*xup3)-xq/(4d0*xup4))
     >      -6d0*b1b0*b2b0*(1d0/(16d0*xup4)-xq/(25d0*xup5)
     >      +xuplog/(4d0*xup4)-(xq*xuplog)/(5d0*xup5))
     >      +3d0*b1b02*b2b0*(2d0/(125d0*xup5)-xq/(108d0*xup6)
     >      +(2d0*xuplog)/(25d0*xup5)-(xq*xuplog)/(18d0*xup6)
     >      +xuplog2/(5d0*xup5)-(xq*xuplog2)/(6d0*xup6))
     >      +6d0*b1b02*b2b0*(-28d0/(125d0*xup5)+(5d0*xq)/(27d0*xup6)
     >      -(3d0*xuplog)/(25d0*xup5)+(xq*xuplog)/(9d0*xup6)
     >      +xuplog2/(5d0*xup5)-(xq*xuplog2)/(6d0*xup6))
     >      -6d0*b1b03*b2b0*(-7d0/(216d0*xup6)
     >      +(57d0*xq)/(2401d0*xup7)
     >      -(7d0*xuplog)/(36d0*xup6)+(57d0*xq*xuplog)/(343d0*xup7)
     >      -xuplog2/(12d0*xup6)+(4d0*xq*xuplog2)/(49d0*xup7)
     >      +xuplog3/(6d0*xup6)-(xq*xuplog3)/(7d0*xup7))
     >      +3d0*b1b04*b2b0*(2929d0/(16807d0*xup7)
     >      -(615d0*xq)/(4096d0*xup8)
     >      +(528d0*xuplog)/(2401d0*xup7)
     >      -(103d0*xq*xuplog)/(512d0*xup8)
     >      -(79d0*xuplog2)/(343d0*xup7)
     >      +(25d0*xq*xuplog2)/(128d0*xup8)
     >      -(10d0*xuplog3)/(49d0*xup7)+(3d0*xq*xuplog3)/(16d0*xup8)
     >      +xuplog4/(7d0*xup7)-(xq*xuplog4)/(8d0*xup8))
     >      +3d0*b2b02*(1d0/(5d0*xup5)-xq/(6d0*xup6))
     >      -3d0*b1b0*b2b02*(1d0/(36d0*xup6)-xq/(49d0*xup7)
     >      +xuplog/(6d0*xup6)-(xq*xuplog)/(7d0*xup7))
     >      +3d0*b1b02*b2b02*(-54d0/(343d0*xup7)
     >      +(35d0*xq)/(256d0*xup8)
     >      -(5d0*xuplog)/(49d0*xup7)+(3d0*xq*xuplog)/(32d0*xup8)
     >      +xuplog2/(7d0*xup7)-(xq*xuplog2)/(8d0*xup8)))
     >      -G2/(2d0*b0*b0*b0)*(
     >      -b1b03*(3d0/(128d0*xlow4)-(6d0*xq)/(625d0*xlow5)
     >      +(3d0*xlowlog)/(32d0*xlow4)
     >      -(6d0*xq*xlowlog)/(125d0*xlow5)
     >      +(3d0*xlowlog2)/(16d0*xlow4)
     >      -(3d0*xq*xlowlog2)/(25d0*xlow5)
     >      +xlowlog3/(4d0*xlow4)-(xq*xlowlog3)/(5d0*xlow5))
     >      +b1b06*(-159580d0/(823543d0*xlow7)
     >      +(21703d0*xq)/(131072d0*xlow8)
     >      -(41931d0*xlowlog)/(117649d0*xlow7)
     >      +(5319d0*xq*xlowlog)/(16384d0*xlow8)
     >      +(4245d0*xlowlog2)/(16807d0*xlow7)
     >      -(825d0*xq*xlowlog2)/(4096d0*xlow8)
     >      +(1415d0*xlowlog3)/(2401d0*xlow7)
     >      -(275d0*xq*xlowlog3)/(512d0*xlow8)
     >      -(75d0*xlowlog4)/(343d0*xlow7)
     >      +(45d0*xq*xlowlog4)/(256d0*xlow8)
     >      -(15d0*xlowlog5)/(49d0*xlow7)
     >      +(9d0*xq*xlowlog5)/(32d0*xlow8)+xlowlog6/(7d0*xlow7)
     >      -(xq*xlowlog6)/(8d0*xlow8))
     >      +b2b03*(1d0/(7d0*xlow7)-xq/(8d0*xlow8))
     >      +1d0/xlow-xq/(2d0*xlow2)
     >      -3d0*b1b0*(1d0/(4d0*xlow2)
     >      -xq/(9d0*xlow3)+xlowlog/(2d0*xlow2)
     >      -(xq*xlowlog)/(3d0*xlow3))
     >      +3d0*b1b02*(2d0/(27d0*xlow3)-xq/(32d0*xlow4)
     >      +(2d0*xlowlog)/(9d0*xlow3)-(xq*xlowlog)/(8d0*xlow4)
     >      +xlowlog2/(3d0*xlow3)-(xq*xlowlog2)/(4d0*xlow4))
     >      +3d0*b1b02*(-10d0/(27d0*xlow3)+(9d0*xq)/(32d0*xlow4)
     >      -xlowlog/(9d0*xlow3)+(xq*xlowlog)/(8d0*xlow4)
     >      +xlowlog2/(3d0*xlow3)-(xq*xlowlog2)/(4d0*xlow4))
     >      -6d0*b1b03*(-9d0/(128d0*xlow4)+(29d0*xq)/(625d0*xlow5)
     >      -(9d0*xlowlog)/(32d0*xlow4)
     >      +(29d0*xq*xlowlog)/(125d0*xlow5)
     >      -xlowlog2/(16d0*xlow4)+(2d0*xq*xlowlog2)/(25d0*xlow5)
     >      +xlowlog3/(4d0*xlow4)-(xq*xlowlog3)/(5d0*xlow5))
     >      +3d0*b1b04*(-56d0/(3125d0*xlow5)+(7d0*xq)/(648d0*xlow6)
     >      -(56d0*xlowlog)/(625d0*xlow5)
     >      +(7d0*xq*xlowlog)/(108d0*xlow6)
     >      -(28d0*xlowlog2)/(125d0*xlow5)
     >      +(7d0*xq*xlowlog2)/(36d0*xlow6)-xlowlog3/(25d0*xlow5)
     >      +(xq*xlowlog3)/(18d0*xlow6)+xlowlog4/(5d0*xlow5)
     >      -(xq*xlowlog4)/(6d0*xlow6))
     >      +3d0*b1b04*(789d0/(3125d0*xlow5)-(67d0*xq)/(324d0*xlow6)
     >      +(164d0*xlowlog)/(625d0*xlow5)
     >      -(13d0*xq*xlowlog)/(54d0*xlow6)
     >      -(43d0*xlowlog2)/(125d0*xlow5)
     >      +(5d0*xq*xlowlog2)/(18d0*xlow6)
     >      -(6d0*xlowlog3)/(25d0*xlow5)
     >      +(2d0*xq*xlowlog3)/(9d0*xlow6)
     >      +xlowlog4/(5d0*xlow5)-(xq*xlowlog4)/(6d0*xlow6))
     >      -3d0*b1b05*(37d0/(972d0*xlow6)
     >      -(3263d0*xq)/(117649d0*xlow7)
     >      +(37d0*xlowlog)/(162d0*xlow6)
     >      -(3263d0*xq*xlowlog)/(16807d0*xlow7)
     >      +(5d0*xlowlog2)/(27d0*xlow6)
     >      -(431d0*xq*xlowlog2)/(2401d0*xlow7)
     >      -(8d0*xlowlog3)/(27d0*xlow6)
     >      +(85d0*xq*xlowlog3)/(343d0*xlow7)
     >      -(7d0*xlowlog4)/(36d0*xlow6)
     >      +(9d0*xq*xlowlog4)/(49d0*xlow7)
     >      +xlowlog5/(6d0*xlow6)-(xq*xlowlog5)/(7d0*xlow7))
     >      +3d0*b2b0*(1d0/(3d0*xlow3)-xq/(4d0*xlow4))
     >      -6d0*b1b0*b2b0*(1d0/(16d0*xlow4)-xq/(25d0*xlow5)
     >      +xlowlog/(4d0*xlow4)-(xq*xlowlog)/(5d0*xlow5))
     >      +3d0*b1b02*b2b0*(2d0/(125d0*xlow5)-xq/(108d0*xlow6)
     >      +(2d0*xlowlog)/(25d0*xlow5)-(xq*xlowlog)/(18d0*xlow6)
     >      +xlowlog2/(5d0*xlow5)-(xq*xlowlog2)/(6d0*xlow6))
     >      +6d0*b1b02*b2b0*(-28d0/(125d0*xlow5)
     >      +(5d0*xq)/(27d0*xlow6)
     >      -(3d0*xlowlog)/(25d0*xlow5)+(xq*xlowlog)/(9d0*xlow6)
     >      +xlowlog2/(5d0*xlow5)-(xq*xlowlog2)/(6d0*xlow6))
     >      -6d0*b1b03*b2b0*(-7d0/(216d0*xlow6)
     >      +(57d0*xq)/(2401d0*xlow7)
     >      -(7d0*xlowlog)/(36d0*xlow6)
     >      +(57d0*xq*xlowlog)/(343d0*xlow7)
     >      -xlowlog2/(12d0*xlow6)+(4d0*xq*xlowlog2)/(49d0*xlow7)
     >      +xlowlog3/(6d0*xlow6)-(xq*xlowlog3)/(7d0*xlow7))
     >      +3d0*b1b04*b2b0*(2929d0/(16807d0*xlow7)
     >      -(615d0*xq)/(4096d0*xlow8)
     >      +(528d0*xlowlog)/(2401d0*xlow7)
     >      -(103d0*xq*xlowlog)/(512d0*xlow8)
     >      -(79d0*xlowlog2)/(343d0*xlow7)
     >      +(25d0*xq*xlowlog2)/(128d0*xlow8)
     >      -(10d0*xlowlog3)/(49d0*xlow7)
     >      +(3d0*xq*xlowlog3)/(16d0*xlow8)
     >      +xlowlog4/(7d0*xlow7)-(xq*xlowlog4)/(8d0*xlow8))
     >      +3d0*b2b02*(1d0/(5d0*xlow5)-xq/(6d0*xlow6))
     >      -3d0*b1b0*b2b02*(1d0/(36d0*xlow6)-xq/(49d0*xlow7)
     >      +xlowlog/(6d0*xlow6)-(xq*xlowlog)/(7d0*xlow7))
     >      +3d0*b1b02*b2b02*(-54d0/(343d0*xlow7)
     >       +(35d0*xq)/(256d0*xlow8)
     >      -(5d0*xlowlog)/(49d0*xlow7)
     >      +(3d0*xq*xlowlog)/(32d0*xlow8)
     >      +xlowlog2/(7d0*xlow7)-(xq*xlowlog2)/(8d0*xlow8)))

      funcspNNLLh = Cgnc0 + Cgnc1 + CG0 + CG1 + CG2

      return
      end

c--------------------------------------------------------------
c     EVOLUTION OF THE HARD PART
c     Integrand to solve analytically:
c     \int_{Qi}^{Qf} [ A \ln(Q^2/mu^2) + B  ]
c     ord=0,1,2 for LL,NLL,NNLL
c--------------------------------------------------------------
      function funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
      real*8 funcsph,xq,xup,xuplog,xlow,xlowlog
      real*8 funcspLLh,funcspNLLh,funcspNNLLh
      integer ord
      common /iorder/ord
      external funcspLLh,funcspNLLh,funcspNNLLh

      if(ord.eq.0) then
        funcsph=funcspLLh(xq,xup,xuplog,xlow,xlowlog,nf)
      elseif(ord.eq.1) then
        funcsph=funcspNLLh(xq,xup,xuplog,xlow,xlowlog,nf)
      elseif(ord.eq.2) then
        funcsph=funcspNNLLh(xq,xup,xuplog,xlow,xlowlog,nf)
      endif

      return
      end

c--------------------------------------------------------------
c     EVOLUTION OF THE HARD PART
c     \exp[ \int_{Qi}^{Qf} ( A \ln(Q^2/mu^2) + B ) ]
c     Inputs: Qi,Qf
c     The integrand is taken consistently at LL,NLL,NNLL depending on
c     the value of the common variable ord: 0,1,2 for LL,NLL,NNLL
c     Output: Revo
c--------------------------------------------------------------
      subroutine expgammaFh(Qi,Qf,Revo)
      implicit none
      real*8 Qi,Qf,Revo
      real*8 funcsph
      integer nf
      real*8 xq,xup,xlow,xuplog,xlowlog,sp1,sp2,sp3,sp,lambda
      real*8 c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5
      common /cons/c0,c02,pi,CF,CA,mc,mb,lambda3,lambda4,lambda5

      if(Qi.le.mc) then
        if(Qf.le.mc) then
            nf = 3  ! Integrate from Qi to Qf
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
        elseif(Qf.gt.mc.and.Qf.le.mb) then
            nf = 3  ! Integrate from Qi to mc
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0 * dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mc to Qf
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        elseif(Qf.gt.mb) then
            nf = 3  ! Integrate from Qi to mc
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mc to mb
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 5  ! Integrate from mb to Qf
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp3 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2 + sp3
        endif
      elseif(Qi.gt.mc.and.Qi.le.mb) then
        if(Qf.le.mc) then
            nf = 4  ! Integrate from Qi to mc
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 3  ! Integrate from mc to Qf
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        elseif(Qf.gt.mc.and.Qf.le.mb) then
            nf = 4  ! Integrate from Qi to Qf
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
        elseif(Qf.gt.mb) then
            nf = 4  ! Integrate from Qi to mb
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 5  ! Integrate from mb to Qf
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        endif
      elseif(Qi.gt.mb) then
        if(Qf.le.mc) then
            nf = 5  ! Integrate from Qi to mb
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mb to mc
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mc/lambda)
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 3  ! Integrate from mc to Qf
            lambda = lambda3
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mc/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp3 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2 + sp3
        elseif(Qf.gt.mc.and.Qf.le.mb) then
            nf = 5  ! Integrate from Qi to mb
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = 2d0*dlog(mb/lambda)
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp1 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            nf = 4  ! Integrate from mb to Qf
            lambda = lambda4
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(mb/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp2 = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
            sp = sp1 + sp2
        elseif(Qf.gt.mb) then
            nf = 5  ! Integrate from Qi to Qf
            lambda = lambda5
            xq = 2d0*dlog(Qf/lambda)
            xup = xq
            xlow = 2d0*dlog(Qi/lambda)
            xuplog = dlog(xup)
            xlowlog = dlog(xlow)
            sp = funcsph(xq,xup,xuplog,xlow,xlowlog,nf)
        endif
      endif

      Revo = dexp( sp )

      return
      end



c--------------------------------------------------------------------------
c               / xf
c               |
c       value = | dx func(x)         n > 1, number of divisions for xf-xi
c               |
c              / xi
c--------------------------------------------------------------------------

      function qgauss(func,xi,xf,n)
      implicit none
      real*8 qgauss,func,xi,xf,xn,value,x1,x2,val
      integer i,n
      external func

      if(n.le.1) then           ! same as n=1
         x1=xi
         x2=xf
         call gauss(func,x1,x2,val)
c         write(6,*) ' qgaus first: ',x1,x2,val
         qgauss=val
         return
      endif

      xn=(xf-xi)/float(n)
      value=0.d0
      x2=xi
      Do i=1,n
         x1=x2
         x2=x1+xn
         call gauss(func,x1,x2,val)
c         write(6,*) ' qgaus sec: ',i,x1,x2,val
         value=value+val
      end do

      qgauss=value

      return
      end

c-------------------------------------------------
      subroutine gauss(func,xi,xf,value)
      implicit none
      real*8 func,xi,xf,value,xm,xr,dx,x(8),w(8)
      real*8 eps
      integer j
      data eps /1.0d-25/
      data w
     1   / 0.0271524594117540948517805725D0,
     2     0.0622535239386478928628438370D0,
     3     0.0951585116824927848099251076D0,
     4     0.1246289712555338720524762822D0,
     5     0.1495959888165767320815017305D0,
     6     0.1691565193950025381893120790D0,
     7     0.1826034150449235888667636680D0,
     8     0.1894506104550684962853967232D0 /
      DATA X
     1   / 0.9894009349916499325961541735D0,
     2     0.9445750230732325760779884155D0,
     3     0.8656312023878317438804678977D0,
     4     0.7554044083550030338951011948D0,
     5     0.6178762444026437484466717640D0,
     6     0.4580167776572273863424194430D0,
     7     0.2816035507792589132304605015D0,
     8     0.0950125098376374401853193354D0 /


      xm=0.5d0*(xf+xi)
      xr=0.5d0*(xf-xi)
      if (abs(xr).lt.eps) print *,
     $     'WARNING: Too high accuracy required for QGAUSS!'
 
      value=0.d0
c      write(6,*) ' gauss ', xm,xr

      Do j=1,8
         dx=xr*x(j)
         value=value+w(j)*(func(xm+dx)+func(xm-dx))
c         write(6,*) ' gauss loop ',j,w(j),dx,xm
      enddo
	
      value=xr*value

      return
      end




