// -*- C++ -*-
// main handler for TMDs
#include "tmdlib/TMDlib.h"
#include "TMDlibPDFpath.h"

using namespace TMDlib;


/*!
 * TMDinit 2nd part
 */
void TMD::TMDinit(const std::string name, int irep ) {

    //cout << " TMDinit2 " << name << endl;

    TMDinfo(name) ;
    // check whether irep is within allowed range
    if (irep <= TMD::TMDgetNumMembers() ) {
        iset = TMD::TMDnumberPDF(name) + irep ;
        if( irep < 10 )  { TMD_Mem = "000"+std::to_string(irep);}
        else if ( irep < 100 )  { TMD_Mem = "00"+std::to_string(irep);}
        else if ( irep < 1000 )  { TMD_Mem = "0"+std::to_string(irep);}
        // cout << " test " << irep << " " << TMD_Mem << endl;

        // TMD_Mem = irep ;
        if ( TMDverbosity() == 2) std::cout << " TMDinit 2: name = " << name << " iset = " << iset << std::endl;
        if ( TMDverbosity() == 2) std::cout << " TMDinit 2: irep = " << irep << " out of " << TMD::TMDgetNumMembers() << std::endl;
    } else {
        std::cout << " Error: TMDinit 2: name = " << name << " irep = " << irep << " not found "<< std::endl;
        std::cout << " Error: TMDinit 2: available Nr of sets = " << TMD::TMDgetNumMembers() << std::endl;
    }
    first=true;
    ncallTMDdensity = 0;
}
/*!
 * TMDinit 3rd part
 */
void TMD::TMDinit(const std::string name) {

    //cout << " TMDinit3 " << name << endl;

    TMDinfo(name) ;

    iset = TMD::TMDnumberPDF(name) ;
    irep = 0 ;
    imode = 0;
    // cout << " TMDinit 3: name = " << name <<endl;
    // cout << " TMDinit 3: pdfpath = " << pdfpath << endl;
    first=true;
    ncallTMDdensity = 0;
}

/*!
 * TMDinit 4th part
 */

void TMD::TMDinit(int & isetInput) {
    //cout << " TMDinit iset " << isetInput << " test fmod " << fmod(isetInput, 100)  << endl;
    // extract base name for number
    int irep = fmod(isetInput, 100);
    int isetBase = isetInput - irep ;
    //cout << " TMDinit isetBase " << isetBase << endl;
    std::string name = TMD::TMDstringPDF(isetBase) ;
    TMDinfo(name) ;
    int ibase=TMD::TMDnumberPDF(name);
    iset = 0 ;
    imode = 0;

    if ( isetInput <= ibase + TMD::TMDgetNumMembers() ) {
        iset = isetInput ;

        if ( irep <= TMD::TMDgetNumMembers() ) {
            if ( TMDverbosity() == 2) std::cout << " TMDinit 4: iset = "<< iset << " name " << name << " irep = " << irep <<std::endl;
        }
    }
    else {
        std::cout << " Error: TMDinit : iset = "<< iset << " name " << name << " isetInput = " << isetInput <<std::endl;
        std::cout << " Error: TMDinit : available Nr of sets = " << TMD::TMDgetNumMembers() << std::endl;
    }
    first=true;
    ncallTMDdensity = 0;
}

/*!
 * TMDpdf 1st
 */
std::vector<double> TMD::TMDpdf( double x, double xbar, double kt, double mu) {
    double top,bottom,charm,strange,uval,dval,gluon, photon, z0, wplus, wminus, higgs ;
    double tbar, ubar, dbar, sbar, cbar, bbar, up, dn, sea;
    //Initialize output

    uval   = 0.;
    dval   = 0.;
    strange = 0.;
    sea = 0. ;
    charm  = 0.;
    bottom = 0.;
    top = 0;
    gluon  = 0.;
    dbar = 0;
    ubar = 0;
    sbar = 0;
    cbar = 0;
    bbar = 0;
    photon = 0;
    std::vector<double> xpq(14);
    for (int j=0; j<=13; j++) {
        xpq[j] =0;
    }
    //cout<< " TMDpdf1 x:" << TMDgetXmin()<< " " << TMDgetXmax() << endl;
    //cout<< " TMDpdf1 q2:" << TMDgetQ2min()<< " " << TMDgetQ2max() << endl;
    if ( x <= TMDgetXmin() || x >= TMDgetXmax() )  {
        std::cout<< " TMDpdf1 x:" << TMDgetXmin()<< " " << TMDgetXmax() << std::endl;
        return xpq;
    }
    if ( mu <= TMDgetQmin() || mu >= TMDgetQmax() ) {
        std::cout<< " TMDpdf1 q = :"<< mu<<" "  << TMDgetQmin()<< " " << TMDgetQmax() << std::endl;
        return xpq;
    }
    // cout << " TMDpdf 1st " << endl;
    int kf =2212 ; /* kf = 2212 for proton */

    if ( x <= TMDgetXmin() ) x=TMDgetXmin();
    if ( x >= TMDgetXmax() ) x=TMDgetXmax();
    if ( mu <= TMDgetQmin() ) mu=TMDgetQmin();
    if ( mu >= TMDgetQmax() ) mu=TMDgetQmax();

    // cout << " calling TMDdensity from TMDpdf1 " << endl;
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon, z0, wplus, wminus, higgs);

    uval = up-ubar ;
    dval = dn-dbar ;

    xpq[0] = tbar ;
    xpq[1] = bbar ;
    xpq[2] = cbar ;
    xpq[3] = sbar ;
    xpq[4] = ubar ;
    xpq[5] = dbar ;
    xpq[6] = gluon ;
    xpq[7] = dval+dbar ;
    xpq[8] = uval+ubar ;
    xpq[9] = strange ;
    xpq[10] = charm ;
    xpq[11] = bottom ;
    xpq[12] = top ;
    xpq[13] = photon ;
    return xpq ;
}


/*!
 * TMDpdf 3rd
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, std::vector<double>& xpq) {
    double top,bottom,charm,sea,uval,dval,gluon,strange, photon, z0, wplus, wminus, higgs ;
    double up, dn, dbar, ubar, sbar, cbar, bbar, tbar;

    //std::vector<double> xpq(13);

    //Initialize output
    uval   = 0.;
    ubar   = 0.;
    dval   = 0.;
    dbar   = 0.;
    strange= 0.;
    sbar   = 0.;
    charm  = 0.;
    cbar   = 0.;
    bottom = 0.;
    bbar   = 0.;
    top = 0;
    tbar = 0.;
    gluon  = 0.;
    photon  = 0.;
    for (int j=0; j<=13; j++) {
        xpq[j] =0;
    }
    if ( x < TMDgetXmin() || x > TMDgetXmax() )  {
        std::cout<< " TMDpdf x-limit: x_min = " << TMDgetXmin()<< " x_max = " << TMDgetXmax() << " x = " << x << std::endl;
        return ;
    }
    if ( mu < TMDgetQmin() || mu > TMDgetQmax() ) {
        std::cout<< " TMDpdf mu-limit: mu_min = :" << TMDgetQmin()<< " mu_max = " << TMDgetQmax() << " mu = "  << mu << std::endl;
        return ;
    }

    // cout << " calling TMDdensity from TMDpdf3 " << endl;
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom,bbar, gluon, photon, z0, wplus, wminus, higgs);


    uval = up-ubar ;
    dval = dn-dbar ;

    xpq[0] = tbar ;
    xpq[1] = bbar ;
    xpq[2] = cbar ;
    xpq[3] = sbar;
    xpq[4] = ubar ;
    xpq[5] = dbar ;
    xpq[6] = gluon ;
    xpq[7] = dval+dbar ;
    xpq[8] = uval+ubar ;
    xpq[9] = strange ;
    xpq[10] = charm ;
    xpq[11] = bottom ;
    xpq[12] = top ;
    xpq[13] = photon ;
}


/*!
 * TMDpdf 5th
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon, double& photon) {

    double dbar, ubar, strange, sbar, cbar, bbar, top, tbar, z0, wplus, wminus, higgs ;
    double up, dn;
    uval   = 0.;
    dval   = 0.;
    charm  = 0.;
    bottom = 0.;
    gluon  = 0.;
    photon  = 0.;
    // cout << " calling TMDdensity from TMDpdf5 " << endl;
    if ( x <= TMDgetXmin() || x >= TMDgetXmax() )  {
        std::cout<< " TMDpdf5 x:" << TMDgetXmin()<< " " << TMDgetXmax() << std::endl;
        return ;
    }
    if ( mu <= TMDgetQmin() || mu >= TMDgetQmax() ) {
        std::cout<< " TMDpdf5 q2 = :"<< mu<<" "  << TMDgetQmin()<< " " << TMDgetQmax() << std::endl;
        return ;
    }
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon, z0, wplus, wminus, higgs);

    uval = up-ubar ;
    dval = dn-dbar ;
    sea = sbar ;

}

/*!
 * TMDpdf 7th
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm,double& cbar, double& bottom,double& bbar, double& gluon, double& photon) {
    double z0, wplus, wminus, higgs;
    // cout << " calling TMDdensity from TMDpdf7 " << iset << endl;
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon, z0, wplus, wminus, higgs);
}
/*!
 * TMDpdf 9th
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm,double& cbar, double& bottom,double& bbar, double& gluon, double& photon, double& z0, double& wplus,  double& wminus, double& higgs) {

    // cout << " calling TMDdensity from TMDpdf7 " << iset << endl;
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon, z0, wplus, wminus, higgs);
}

/*!
 * TMDpdf 11th
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm,double& cbar, double& bottom,double& bbar, double& top, double& tbar, double& gluon, double& photon, double& z0, double& wplus,  double& wminus, double& higgs) {

    // cout << " calling TMDdensity from TMDpdf7 " << iset << endl;
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, top, tbar, gluon, photon, z0, wplus, wminus, higgs);
}
/*!
 *   Main routine for TMDdensities
 */
void TMD::TMDdensity( double x, double xbar, double kt, double mu, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm,double& cbar, double& bottom,double& bbar, double& gluon, double& photon, double& z0, double&  wplus,  double& wminus,  double& higgs ) {

    //Initialize output
    double uval, dval, top,tbar, sea;
    uval   = 0.;
    ubar   = 0.;
    dval   = 0.;
    dbar   = 0.;
    strange= 0.;
    sea    = 0.;
    sbar   = 0.;
    charm  = 0.;
    cbar   = 0.;
    bottom = 0.;
    bbar   = 0.;
    gluon  = 0.;
    photon  = 0.;
    // cout <<" TMDdensit TMDSet Description: " << TMDgetIndex() << endl;
    //cout<< " TMDdensity x:" << TMDgetXmin()<< " " << TMDgetXmin() << endl;
    //cout<< " TMDdensity q:" << TMDgetQmin()<< " " << TMDgetQmin() << endl;
    if ( x < TMDgetXmin() || x > TMDgetXmax() )  {
        ncallTMDdensity=ncallTMDdensity + 1 ;
        if (TMDgetExtrapolation_kt() == "fixed" ) {
            if ( x < TMDgetXmin() )
            { x = TMDgetXmin(); }
            else {x = TMDgetXmax();}
            if (ncallTMDdensity <=10 ) {std::cout<< " TMDdensity x: fixed extraploation used for TMDset "  << TMDgetIndex() << std::endl;}
        }
        else {
            if (ncallTMDdensity <=10 ) {
                std::cout<< " TMDdensity x: no extraploation requested -> return 0 " << TMDgetXmin()<< " " << TMDgetXmax() <<" for TMDset "  << TMDgetIndex() << std::endl;
            }
            return ;
        }
    }
    if ( mu < TMDgetQmin() || mu > TMDgetQmax() ) {
        ncallTMDdensity=ncallTMDdensity + 1 ;
        if (TMDgetExtrapolation_Q2() == "fixed" ) {
            if ( mu < TMDgetQmin() )
            { mu = TMDgetQmin(); }
            else {mu = TMDgetQmax();}
            if (ncallTMDdensity <=10 ) {std::cout<< " TMDdensity q2: fixed extraploation used " << mu<< " " <<TMDgetQmin() <<" " << TMDgetQmax() <<" for TMDset "  << TMDgetIndex()  << std::endl;}
        }
        else {
            if (ncallTMDdensity <=10 ) {
                std::cout<< " TMDdensity q2: no extraploation requested -> return 0 "<< mu <<" "  << sqrt(TMDgetQ2min())<< " " << sqrt(TMDgetQ2max()) <<" for TMDset "  << TMDgetIndex() << std::endl;
            }
            return ;
        }
    }
    //cout << " checking on format " << NewFormat << endl;
    if (NewFormat == 1 ) {
        //cout << "TMDdensity: TMDscheme = " << TMDgetScheme() << endl;
        if (  TMDgetScheme() == "PB TMD-EW" ) {
            //cout << " TMDdensity:  PB TMD scheme selected iset = " << iset << endl ;
            allFlavuPDFew(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon, z0, wplus, wminus, higgs ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if (  TMDgetScheme() == "PB TMD" ) {
            if ( TMDverbosity() == 2) std::cout << " TMDdensity:  PB TMD scheme selected iset = " << iset << std::endl ;
            allFlavuPDF(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if (  TMDgetScheme() == "CCFM" ) {
            if ( TMDverbosity() == 2)  std::cout << " TMDdensity:  CCFM scheme selected iset = " << iset << std::endl ;
            ccfm_gluon(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if (  TMDgetScheme() == "KS" ) {
            if ( TMDverbosity() == 2) std::cout << " TMDdensity:  KS scheme selected iset = " << iset << std::endl ;
            ksPDF(x, kt, mu, up, dn, sea, charm, bottom, gluon ) ;
            uval = up-sea ;
            dval = dn-sea ;
        }
        else if (  TMDgetScheme() == "Pavia TMDs" ) {
            if ( TMDverbosity() == 2) std::cout << " TMDdensity:  Pavia TMD scheme selected " << std::endl ;
            Pavia(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else { std::cout << " TMDdensity:  no valid scheme selected: EXIT_FAILURE  for TMDset "  << TMDgetIndex() << std::endl; exit (EXIT_FAILURE); }

    }
    else if( NewFormat == 0) {
        if ( TMDverbosity() == 2)  std::cout << " TMDlib: TMDhandler checking on format " << NewFormat << std::endl;
        if ( iset >= 101000 && iset <=109999  ) {
            allFlavuPDF_old(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset >= 800001 && iset <=800010  ) {
            allFlavuPDF_old(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset >= 10000000  ) {
            allFlavuPDFn(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset == 200001  ) {
            gbwuPDFlight(x,kt, gluon) ;
        }
        else if ( iset == 200002  ) {
            gbwuPDFcharm(x,kt, gluon) ;
        }
        else if ( iset == 210001  ) {
            gluon=blueml(&x,&kt,&mu) ;
        }
        else if ( iset >= 300001 && iset <= 300003 ) {
            sbrsPDF(irep, imode, x, kt, mu, uval, dval, sea, charm) ;
            if ( TMDverbosity() == 2)  std::cout << " TMDhandler sbrs PDF " << uval << " " << dval << " " << sea << std::endl ;
            up = uval;
            ubar = 0;
            dn = dval;
            dbar = 0;
            strange = sea;
        }
        else if ( iset >= 400000 && iset <= 400004 ) {
            ksPDF(x, kt, mu, up, dn, sea, charm, bottom, gluon ) ;
            uval = up-sea ;
            dval = dn-sea ;
        }
        /*
        else if ( iset == 400101 ) {
          ksDLCPDF(x, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, gluon ) ;
          uval = up - ubar;
          dval = dn - dbar;
          }
        */
        else if ( iset >= 410000 && iset <= 410100 ) {
            ksBHKSPDF(x, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset >= 500001 && iset <= 500004 ) {
            tmd_ME(x,kt,mu, uval, dval, sea, charm, bottom, gluon ) ;
        }

        else {
            std::cout << " TMDhandler: iset not defined " << iset << std::endl;
        }
    }
    else { std::cout << " TMDhandler: NewFormat not defined " << NewFormat << std::endl ; }


//    cout << " TMDpdf: x, xbar, kt, mu " << x << " " << xbar << " " << kt << " " << mu << endl;

}

/*!
 *   Main routine for TMDdensities
 */
void TMD::TMDdensity( double x, double xbar, double kt, double mu, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm,double& cbar, double& bottom, double& bbar, double& top, double&tbar , double& gluon, double& photon, double& z0, double&  wplus,  double& wminus,  double& higgs ) {

    //Initialize output
    double uval, dval, sea;
    uval   = 0.;
    ubar   = 0.;
    dval   = 0.;
    dbar   = 0.;
    strange= 0.;
    sea    = 0.;
    sbar   = 0.;
    charm  = 0.;
    cbar   = 0.;
    bottom = 0.;
    bbar   = 0.;
    top = 0.;
    tbar   = 0.;
    gluon  = 0.;
    photon  = 0.;
    // cout <<" TMDdensit TMDSet Description: " << TMDgetIndex() << endl;
    //cout<< " TMDdensity x:" << TMDgetXmin()<< " " << TMDgetXmin() << endl;
    //cout<< " TMDdensity q:" << TMDgetQmin()<< " " << TMDgetQmin() << endl;
    if ( x < TMDgetXmin() || x > TMDgetXmax() )  {
        ncallTMDdensity=ncallTMDdensity + 1 ;
        if (TMDgetExtrapolation_kt() == "fixed" ) {
            if ( x < TMDgetXmin() )
            { x = TMDgetXmin(); }
            else {x = TMDgetXmax();}
            if (ncallTMDdensity <=10 ) {std::cout<< " TMDdensity x: fixed extraploation used for TMDset "  << TMDgetIndex() << std::endl;}
        }
        else {
            if (ncallTMDdensity <=10 ) {
                std::cout<< " TMDdensity x: no extraploation requested -> return 0 " << TMDgetXmin()<< " " << TMDgetXmax() <<" for TMDset "  << TMDgetIndex() << std::endl;
            }
            return ;
        }
    }
    if ( mu < TMDgetQmin() || mu > TMDgetQmax() ) {
        ncallTMDdensity=ncallTMDdensity + 1 ;
        if (TMDgetExtrapolation_Q2() == "fixed" ) {
            if ( mu < TMDgetQmin() )
            { mu = TMDgetQmin(); }
            else {mu = TMDgetQmax();}
            if (ncallTMDdensity <=10 ) {std::cout<< " TMDdensity q2: fixed extraploation used " << mu<< " " <<TMDgetQmin() <<" " << TMDgetQmax() <<" for TMDset "  << TMDgetIndex()  << std::endl;}
        }
        else {
            if (ncallTMDdensity <=10 ) {
                std::cout<< " TMDdensity q2: no extraploation requested -> return 0 "<< mu <<" "  << sqrt(TMDgetQ2min())<< " " << sqrt(TMDgetQ2max()) <<" for TMDset "  << TMDgetIndex() << std::endl;
            }
            return ;
        }
    }
    //cout << " checking on format " << NewFormat << endl;
    if (NewFormat == 1 ) {
        //cout << "TMDdensity: TMDscheme = " << TMDgetScheme() << endl;
        if (  TMDgetScheme() == "PB TMD-EW" ) {
            //cout << " TMDdensity:  PB TMD scheme selected iset = " << iset << endl ;
            allFlavuPDFew(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon, z0, wplus, wminus, higgs ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if (  TMDgetScheme() == "PB TMD" ) {
            if ( TMDverbosity() == 2) std::cout << " TMDdensity:  PB TMD scheme selected iset = " << iset << std::endl ;
            allFlavuPDF(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if (  TMDgetScheme() == "CCFM" ) {
            if ( TMDverbosity() == 2)  std::cout << " TMDdensity:  CCFM scheme selected iset = " << iset << std::endl ;
            ccfm_gluon(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if (  TMDgetScheme() == "KS" ) {
            if ( TMDverbosity() == 2) std::cout << " TMDdensity:  KS scheme selected iset = " << iset << std::endl ;
            ksPDF(x, kt, mu, up, dn, sea, charm, bottom, gluon ) ;
            uval = up-sea ;
            dval = dn-sea ;
        }
        else if (  TMDgetScheme() == "Pavia TMDs" ) {
            if ( TMDverbosity() == 2) std::cout << " TMDdensity:  Pavia TMD scheme selected " << std::endl ;
            Pavia(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else { std::cout << " TMDdensity:  no valid scheme selected: EXIT_FAILURE  for TMDset "  << TMDgetIndex() << std::endl; exit (EXIT_FAILURE); }

    }
    else if( NewFormat == 0) {
        if ( TMDverbosity() == 2)  std::cout << " TMDlib: TMDhandler checking on format " << NewFormat << std::endl;
        if ( iset >= 101000 && iset <=109999  ) {
            allFlavuPDF_old(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset >= 800001 && iset <=800010  ) {
            allFlavuPDF_old(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset >= 10000000  ) {
            allFlavuPDFn(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, photon,  gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset == 200001  ) {
            gbwuPDFlight(x,kt, gluon) ;
        }
        else if ( iset == 200002  ) {
            gbwuPDFcharm(x,kt, gluon) ;
        }
        else if ( iset == 210001  ) {
            gluon=blueml(&x,&kt,&mu) ;
        }
        else if ( iset >= 300001 && iset <= 300003 ) {
            sbrsPDF(irep, imode, x, kt, mu, uval, dval, sea, charm) ;
            if ( TMDverbosity() == 2)  std::cout << " TMDhandler sbrs PDF " << uval << " " << dval << " " << sea << std::endl ;
            up = uval;
            ubar = 0;
            dn = dval;
            dbar = 0;
            strange = sea;
        }
        else if ( iset >= 400000 && iset <= 400004 ) {
            ksPDF(x, kt, mu, up, dn, sea, charm, bottom, gluon ) ;
            uval = up-sea ;
            dval = dn-sea ;
        }
        /*
        else if ( iset == 400101 ) {
          ksDLCPDF(x, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, gluon ) ;
          uval = up - ubar;
          dval = dn - dbar;
          }
        */
        else if ( iset >= 410000 && iset <= 410100 ) {
            ksBHKSPDF(x, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, gluon ) ;
            uval = up-ubar ;
            dval = dn-dbar ;
        }
        else if ( iset >= 500001 && iset <= 500004 ) {
            tmd_ME(x,kt,mu, uval, dval, sea, charm, bottom, gluon ) ;
        }

        else {
            std::cout << " TMDhandler: iset not defined " << iset << std::endl;
        }
    }
    else { std::cout << " TMDhandler: NewFormat not defined " << NewFormat << std::endl ; }


//    cout << " TMDpdf: x, xbar, kt, mu " << x << " " << xbar << " " << kt << " " << mu << endl;

}


