#include "tmdlib/TMDlib.h"
//#include "TMDlibPDFpath.h"

#include <iostream>
#include <sstream>
#include <string>
#include <fstream>

using namespace TMDlib;
/* Common Block Declarations */

//extern int iset, irep, imode ;
//extern int iset, irep ;
//extern int iset ;
extern std::string pdfpath;

//extern bool first;

void TMD::allFlavuPDFn(double x, double kt, double p, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm, double& cbar, double& bottom, double& bbar, double& top, double& tbar, double& phot,  double& glu)
{
    /**
    * Routine to interpolate TMDgrids and to return momentum weighted TMD parton density as function of x, kt, scale p for different flavors.
    *
    * General interpolation routine for PB TMDs (as well as CCFM uPDFs).
    *
    * TMDgrid in form of 51,51,51 for logx, logkt2 and log p, for flavours [-6,..,0,... 6, 22], note id=0->id=21 gluon
    *
    *  Parametrization for nucleons.
    */
    /* Initialized data */

    static int na[3] = { 51,51,51 };


    /* Local static variables */
    /*
        static double a[153],xa[3];
        static double px[52], xx[52], q2x[52];
        static double f_grid0[132651], f_grid1[132651], f_grid2[132651], f_grid3[132651], f_grid4[132651];
        static double f_grid5[132651], f_grid6[132651], f_grid7[132651];
        static double f_grid1m[132651], f_grid2m[132651], f_grid3m[132651], f_grid4m[132651], f_grid5m[132651], f_grid6m[132651];
        static double qscal, qg0, q2;
        static int n1min, n1max, n2min, n2max, n3min, n3max, ncall;
        static int iqqbar, iglu, ikincut, ipgg, ns_sel;
    */
    /* Local variables */
    int irr, i, j, k, in, ip;
    int igrid;
    char txt[72], adum[1];
    double rp,rx,rq2;
    double rkms0, rkms1, rkms2, rkms3, rkms4, rkms5, rkms6, rkms7, rkms1b, rkms2b, rkms3b, rkms4b, rkms5b, rkms6b;
    double bb, ktqt, qcdlam, scal, qtd1, ktd1;
    double uval,dval;

    std::string filename;

    // std::cout << " in allFlavuPDFn " << std::endl;

    /* Function Body */
    if (first) {
        n1min = 0;
        n1max = 0;
        n2min = 0;
        n2max = 0;
        n3min = 0;
        n3max = 0;
        ncall = 0;
        i = 0;
        scal = 1.;
        ipgg = 0;
        rkms0 =0;
        rkms1 =0;
        rkms2=0;
        rkms3=0;
        rkms4=0;
        rkms5=0;
        rkms6=0;
        rkms7=0;
        rkms1b=0;
        rkms2b=0;
        rkms3b=0;
        rkms4b=0;
        rkms5b=0;
        rkms6b=0;
        ns_sel = -1;
        /*      pdfpath = getenv ("TMDlibPATH");
              if (pdfpath!="") { }
                else  {pdfpath="../data/"; }
              if (pdfpath!="") { std::cout << "TMDlib::allFalvupDFn: TMDlibPATH = " <<  pdfpath << std::endl;}
                  else { std::cout << " NO TMDlibPATH  set " <<std::endl;
                  return ;}
        */
        if( iset < 0 )
        {   std::cout << "  TMDlib:allFlavuPDFn set not foud. iset = " << iset << std::endl;
            return ;
        }
        else if ( iset == 10901300 )   {filename=pdfpath+TMD_Dir+"PB-asq.dat";}
        else if ( iset == 10901400 )   {filename=pdfpath+TMD_Dir+"PB-aspt.dat";}
        else if ( iset == 10902300 )   {filename=pdfpath+TMD_Dir+"PB-asq.dat";}
        else if ( iset == 10902301 )   {filename=pdfpath+TMD_Dir+"PB-asq-1.dat";}
        else if ( iset == 10902302 )   {filename=pdfpath+TMD_Dir+"PB-asq-2.dat";}
        else if ( iset == 10902303 )   {filename=pdfpath+TMD_Dir+"PB-asq-3.dat";}
        else if ( iset == 10902304 )   {filename=pdfpath+TMD_Dir+"PB-asq-4.dat";}
        else if ( iset == 10902305 )   {filename=pdfpath+TMD_Dir+"PB-asq-5.dat";}
        else if ( iset == 10902306 )   {filename=pdfpath+TMD_Dir+"PB-asq-6.dat";}
        else if ( iset == 10902307 )   {filename=pdfpath+TMD_Dir+"PB-asq-7.dat";}
        else if ( iset == 10902308 )   {filename=pdfpath+TMD_Dir+"PB-asq-8.dat";}
        else if ( iset == 10902309 )   {filename=pdfpath+TMD_Dir+"PB-asq-9.dat";}
        else if ( iset == 10902310 )   {filename=pdfpath+TMD_Dir+"PB-asq-10.dat";}
        else if ( iset == 10902311 )   {filename=pdfpath+TMD_Dir+"PB-asq-11.dat";}
        else if ( iset == 10902312 )   {filename=pdfpath+TMD_Dir+"PB-asq-12.dat";}
        else if ( iset == 10902313 )   {filename=pdfpath+TMD_Dir+"PB-asq-13.dat";}
        else if ( iset == 10902314 )   {filename=pdfpath+TMD_Dir+"PB-asq-14.dat";}
        else if ( iset == 10902315 )   {filename=pdfpath+TMD_Dir+"PB-asq-15.dat";}
        else if ( iset == 10902316 )   {filename=pdfpath+TMD_Dir+"PB-asq-16.dat";}
        else if ( iset == 10902317 )   {filename=pdfpath+TMD_Dir+"PB-asq-17.dat";}
        else if ( iset == 10902318 )   {filename=pdfpath+TMD_Dir+"PB-asq-18.dat";}
        else if ( iset == 10902319 )   {filename=pdfpath+TMD_Dir+"PB-asq-19.dat";}
        else if ( iset == 10902320 )   {filename=pdfpath+TMD_Dir+"PB-asq-20.dat";}
        else if ( iset == 10902321 )   {filename=pdfpath+TMD_Dir+"PB-asq-21.dat";}
        else if ( iset == 10902322 )   {filename=pdfpath+TMD_Dir+"PB-asq-22.dat";}
        else if ( iset == 10902323 )   {filename=pdfpath+TMD_Dir+"PB-asq-23.dat";}
        else if ( iset == 10902324 )   {filename=pdfpath+TMD_Dir+"PB-asq-24.dat";}
        else if ( iset == 10902325 )   {filename=pdfpath+TMD_Dir+"PB-asq-25.dat";}
        else if ( iset == 10902326 )   {filename=pdfpath+TMD_Dir+"PB-asq-26.dat";}
        else if ( iset == 10902327 )   {filename=pdfpath+TMD_Dir+"PB-asq-27.dat";}
        else if ( iset == 10902328 )   {filename=pdfpath+TMD_Dir+"PB-asq-28.dat";}
        else if ( iset == 10902329 )   {filename=pdfpath+TMD_Dir+"PB-asq-29.dat";}
        else if ( iset == 10902330 )   {filename=pdfpath+TMD_Dir+"PB-asq-30.dat";}
        else if ( iset == 10902331 )   {filename=pdfpath+TMD_Dir+"PB-asq-31.dat";}
        else if ( iset == 10902332 )   {filename=pdfpath+TMD_Dir+"PB-asq-32.dat";}

        else if ( iset == 10902400 )   {filename=pdfpath+TMD_Dir+"PB-aspt.dat";}
        else if ( iset == 10902401 )   {filename=pdfpath+TMD_Dir+"PB-aspt-1.dat";}
        else if ( iset == 10902402 )   {filename=pdfpath+TMD_Dir+"PB-aspt-2.dat";}
        else if ( iset == 10902403 )   {filename=pdfpath+TMD_Dir+"PB-aspt-3.dat";}
        else if ( iset == 10902404 )   {filename=pdfpath+TMD_Dir+"PB-aspt-4.dat";}
        else if ( iset == 10902405 )   {filename=pdfpath+TMD_Dir+"PB-aspt-5.dat";}
        else if ( iset == 10902406 )   {filename=pdfpath+TMD_Dir+"PB-aspt-6.dat";}
        else if ( iset == 10902407 )   {filename=pdfpath+TMD_Dir+"PB-aspt-7.dat";}
        else if ( iset == 10902408 )   {filename=pdfpath+TMD_Dir+"PB-aspt-8.dat";}
        else if ( iset == 10902409 )   {filename=pdfpath+TMD_Dir+"PB-aspt-9.dat";}
        else if ( iset == 10902410 )   {filename=pdfpath+TMD_Dir+"PB-aspt-10.dat";}
        else if ( iset == 10902411 )   {filename=pdfpath+TMD_Dir+"PB-aspt-11.dat";}
        else if ( iset == 10902412 )   {filename=pdfpath+TMD_Dir+"PB-aspt-12.dat";}
        else if ( iset == 10902413 )   {filename=pdfpath+TMD_Dir+"PB-aspt-13.dat";}
        else if ( iset == 10902414 )   {filename=pdfpath+TMD_Dir+"PB-aspt-14.dat";}
        else if ( iset == 10902415 )   {filename=pdfpath+TMD_Dir+"PB-aspt-15.dat";}
        else if ( iset == 10902416 )   {filename=pdfpath+TMD_Dir+"PB-aspt-16.dat";}
        else if ( iset == 10902417 )   {filename=pdfpath+TMD_Dir+"PB-aspt-17.dat";}
        else if ( iset == 10902418 )   {filename=pdfpath+TMD_Dir+"PB-aspt-18.dat";}
        else if ( iset == 10902419 )   {filename=pdfpath+TMD_Dir+"PB-aspt-19.dat";}
        else if ( iset == 10902420 )   {filename=pdfpath+TMD_Dir+"PB-aspt-20.dat";}
        else if ( iset == 10902421 )   {filename=pdfpath+TMD_Dir+"PB-aspt-21.dat";}
        else if ( iset == 10902422 )   {filename=pdfpath+TMD_Dir+"PB-aspt-22.dat";}
        else if ( iset == 10902423 )   {filename=pdfpath+TMD_Dir+"PB-aspt-23.dat";}
        else if ( iset == 10902424 )   {filename=pdfpath+TMD_Dir+"PB-aspt-24.dat";}
        else if ( iset == 10902425 )   {filename=pdfpath+TMD_Dir+"PB-aspt-25.dat";}
        else if ( iset == 10902426 )   {filename=pdfpath+TMD_Dir+"PB-aspt-26.dat";}
        else if ( iset == 10902427 )   {filename=pdfpath+TMD_Dir+"PB-aspt-27.dat";}
        else if ( iset == 10902428 )   {filename=pdfpath+TMD_Dir+"PB-aspt-28.dat";}
        else if ( iset == 10902429 )   {filename=pdfpath+TMD_Dir+"PB-aspt-29.dat";}
        else if ( iset == 10902430 )   {filename=pdfpath+TMD_Dir+"PB-aspt-30.dat";}
        else if ( iset == 10902431 )   {filename=pdfpath+TMD_Dir+"PB-aspt-31.dat";}
        else if ( iset == 10902432 )   {filename=pdfpath+TMD_Dir+"PB-aspt-32.dat";}




        else if ( iset == 10903300 )   {filename=pdfpath+TMD_Dir+"PB-asq.dat";}
        else if ( iset == 10903400 )   {filename=pdfpath+TMD_Dir+"PB-aspt.dat";}
        else if ( iset == 10904300 )   {filename=pdfpath+TMD_Dir+"PB-Lead-asq.dat";}
        else if ( iset == 10904400 )   {filename=pdfpath+TMD_Dir+"PB-Lead-aspt.dat";}
        // std::cout <<  "allFlavuPDFn: read grid from " << filename << std::endl;
        std::string text1,text2,text3,text4,textdum;
        const char *testname = filename.c_str();
        // std::cout << testname << std::endl;
        std::ifstream myfile (testname);
        if (myfile.is_open()) {
            myfile >> text1 >> textdum >> qg0 >> text2 >>  ikincut ;
            // std::cout << " 1st read " << std::endl;
            // std::cout << text1 << " " << qg0 << " "<< text2 << " " << ikincut <<std::endl;
            if( iset != 101000) {
                myfile >> text1 >> textdum >> ipgg >> text2 >> textdum >> ns_sel ;
                // std::cout << " 2nd read " << std::endl;
                // std::cout << text1 << " " << ipgg << " "<< text2 << " " << ns_sel <<std::endl;
            }
            if ( iset >= 101000 && iset < 101003 ) { //std::cout << " 101000 - 101003 read nothing " << std::endl;
            }
            else if ( iset >= 101010 && iset < 101023 ) {
                myfile >> text1 >> textdum >> textdum >> textdum >> textdum>> textdum ;
                // std::cout << " 101010 - 101023 read 3 " << std::endl;
                // std::cout << text1 << " "  <<  textdum << " " << textdum << std::endl;
            }
            else if ( iset >= 101200 && iset < 101320 ) {
                myfile >> text1 >> textdum >> qcdlam  ;
                //std::cout << " 101200 - 101320 read 3 qdclam " << std::endl;
            }
            else if ( iset >= 101111 && iset <= 101119 ) {
                myfile >> text1 >> textdum >> qcdlam  ;
                // std::cout << " 101111 - 101119 read 3 qdclam " << std::endl;
            }
            else if ( iset >= 101500 && iset <= 109999 ) {
                myfile >> text1 >> textdum >> qcdlam  ;
                //std::cout << " 101500 - 109999 read 3 qdclam " << std::endl;
            }
            else if ( iset >= 800001 && iset <= 800010 ) {
                myfile >> text1 >> textdum >> qcdlam  ;
                //std::cout << " 800001 - 800010 read 3 qdclam " << std::endl;
            }
            else if ( iset >= 10000000 && iset <= 19000000 ) {
                myfile >> text1 >> textdum >> qcdlam  ;
                //std::cout << " 800001 - 800010 read 3 qdclam " << std::endl;
            }
            else {
                myfile >> textdum >> textdum >> textdum >> textdum >> textdum >> textdum;
                // std::cout << " remaining read 6 " << std::endl;
            }

            myfile >> text1 >> text2 >> text3>> text4  ;
            //std::cout << " 4th read " << std::endl;
            // std::cout << text1 << text2 << text3 << text4 << std::endl;


            for (j = 1; j <= 51; ++j) {
                for (i = 1; i <= 51; ++i) {
                    for (k = 1; k <= 51; ++k) {
                        if (!myfile.eof() ) {
                            if ( iset >= 101200 && iset < 101320 ) {
                                myfile >>  rx >> rq2 >> rp >> rkms4b >>rkms3b >> rkms2b >> rkms1b
                                       >> rkms0 >> rkms1 >> rkms2 >> rkms3 >> rkms4 ;
                            }
                            else if ( iset >= 101111 && iset <= 101119 ) {
                                myfile >>  rx >> rq2 >>rp >> rkms6b >> rkms5b >> rkms4b >>rkms3b >> rkms2b >> rkms1b >> rkms0 >> rkms1 >> rkms2 >> rkms3 >> rkms4 >> rkms5>> rkms6>> rkms7;
                            }
                            else if ( iset >= 101500 && iset <= 109999 ) {
                                myfile >>  rx >> rq2 >>rp >> rkms6b >> rkms5b >> rkms4b >>rkms3b >> rkms2b >> rkms1b >> rkms0 >> rkms1 >> rkms2 >> rkms3 >> rkms4 >> rkms5>> rkms6>> rkms7;
                            }
                            else if ( iset >= 800001 && iset <= 800010 ) {
                                myfile >>  rx >> rq2 >>rp >> rkms6b >> rkms5b >> rkms4b >>rkms3b >> rkms2b >> rkms1b >> rkms0 >> rkms1 >> rkms2 >> rkms3 >> rkms4 >> rkms5>> rkms6>> rkms7;
                            }
                            else if ( iset >= 10000000 && iset <= 19000000 ) {
                                myfile >>  rx >> rq2 >>rp >> rkms6b >> rkms5b >> rkms4b >>rkms3b >> rkms2b >> rkms1b >> rkms0 >> rkms1 >> rkms2 >> rkms3 >> rkms4 >> rkms5>> rkms6>> rkms7;
                            }
                            else {
                                myfile >>  rx >> rq2 >> rp >> rkms0  ;
                            }
                            // std::cout << " rx " << rx << " " << rq2 << " " << rp << " rkms0 " << rkms0  << " "  << rkms1 << " " <<rkms2 << "rkms3 "<< rkms3 << " rkms4 " << rkms4 << std::endl;
                        }
                        if (rx != rx) rx = 0.;
                        if (rq2 != rq2) rq2 = 0.;
                        if (rp != rp) rp = 0.;
                        if (rkms0 != rkms0) rkms0=0;
                        if (rkms1 != rkms1) rkms1=0;
                        if (rkms2 != rkms2) rkms2=0;
                        if (rkms3 != rkms3) rkms3=0;
                        if (rkms4 != rkms4) rkms4=0;
                        if (rkms5 != rkms5) rkms5=0;
                        if (rkms6 != rkms6) rkms6=0;
                        if (rkms7 != rkms7) rkms7=0;
                        if (rkms1b != rkms1b) rkms1b=0;
                        if (rkms2b != rkms2b) rkms2b=0;
                        if (rkms3b != rkms3b) rkms3b=0;
                        if (rkms4b != rkms4b) rkms4b=0;
                        if (rkms5b != rkms5b) rkms5b=0;
                        if (rkms6b != rkms6b) rkms6b=0;
                        xx[i] = rx;
                        q2x[j] = rq2;
                        px[k] = rp;
                        // std::cout << i << " " << j <<" " << q2x[j] << " " << k << std::endl;
                        f_grid0[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms0) * scal;
                        f_grid1[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms1) * scal;
                        f_grid2[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms2) * scal;
                        f_grid3[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms3) * scal;
                        f_grid4[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms4) * scal;
                        f_grid5[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms5) * scal;
                        f_grid6[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms6) * scal;
                        f_grid7[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms7) * scal;
                        f_grid1m[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms1b) * scal;
                        f_grid2m[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms2b) * scal;
                        f_grid3m[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms3b) * scal;
                        f_grid4m[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms4b) * scal;
                        f_grid5m[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms5b) * scal;
                        f_grid6m[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms6b) * scal;
                        f_grid7[j + (i + k * 51) * 51 - 2653] = std::max(0.,rkms7) * scal;

                    }
                }
            }
            // std::cout << " end of file: allFalvuPDFn at " << j << " " << i << " " << k << std::endl;
        }
        else {
            std::cout << " error file not found " << std::endl;
            return ;
        }
        in = 0;
        for (i = 1; i <= 51; ++i) {
            ++in;
            a[in - 1] = q2x[i];
        }
        for (i = 1; i <= 51; ++i) {
            ++in;
            a[in - 1] = xx[i];
        }
        for (i = 1; i <= 51; ++i) {
            ++in;
            a[in - 1] = px[i];
        }

        first = false;
        // std::cout <<  "  parton densities read from file unit 30 " << std::endl;
    }
    // initialise values to zero
    glu = 0;
    up = 0;
    ubar  = 0 ;
    uval = 0 ;
    dn = 0;
    dbar = 0;
    dval = 0;
    strange = 0;
    sbar = 0;
    charm = 0;
    cbar = 0;
    bottom = 0;
    bbar = 0;
    top = 0;
    tbar = 0;
    phot = 0;



    ++ncall;
    q2 = kt*kt ;
    // std::cout << " params in allFlavuPDFn: " <<x  << " " << kt << " " <<  p << std::endl;

    xa[0] = log(q2);
    xa[1] = log(x);
    xa[2] = log(p);
    // std::cout << " ccmfuPDF p = " << p << std::endl;
    //std::cout << " params in ccmfupfn: " <<xa[0] << " " << xa[1] << " " <<  xa[2] << std::endl;
    //std::cout << " params in ccmfupfn x: " << xa[1] << " limits " << xx[1] << " " << xx[51] <<  std::endl;
    //std::cout << " params in ccmfupfn kt2: " << xa[0] << " limits " << q2x[1] << " " << q2x[51] <<  std::endl;
    //std::cout << " params in ccmfupfn p: " << xa[2] << " limits " << px[1] << " " << px[51] <<  std::endl;
    if (xa[2] < px[1]) {
        ++n3min;
        if (n3min < 10) {
            std::cout <<" TMDlib:allFlavuPDFn p out of range " << p << " min p " << exp(px[1]) <<std::endl;
            //std::cout <<" TMDlib:allFlavuPDFn p out of range " << xa[2] << " " << px[1] << std::endl;
        } else if (n3min == 10) {
            std::cout << " TMDlib:allFlavuPDFn p out of range " << p << " min p " << exp(px[1]) << std::endl;
            std::cout << " TMDlib:allFlavuPDFn last message printed: min p " << std::endl;
        }
        xa[2] = px[1];
    }
    if (xa[2] > px[51]) {
        ++n3max;
        if (n3max < 10) {
            //    write(6,*) ' p out of range ',p,' max p ',exp(px(n3)) */
        } else if (n3max == 10) {
            //    write(6,*) ' p out of range ',p,' max p ',exp(px(n3)) */
            //    write(6,*) ' last message printed: max p' */
        }
        xa[2] = px[51];
    }
    if (xa[1] >= xx[51]) {
        xa[1] = xx[51] - 1e-4;
    }
    if (xa[1] < xx[1]) {
        ++n2min;
        if (n2min < 10) {
            //   write(6,*) '  x out of range ',x,' min ',exp(xx(1)) */
        } else if (n2min == 10) {
            //  write(6,*) '  x out of range ',x,' min ',exp(xx(1)) */
            //   write(6,*) ' last message printed: min x' */
        }
        xa[1] = xx[1];
    }
//    std::cout << " q2 " << q2 << " kt " << kt << " " << exp(q2x[1]) << std::endl;
    if (xa[0] < q2x[1]) {
        ++n1min;
        /*
             if (n1min < 10) {
                 std::cout << "  k2 out of range " << q2 << "  min " << exp(q2x[1]) << std::endl;
        	} else if (n1min == 10) {
                 std::cout << "  k2 out of range " << q2 << "  min " << exp(q2x[1]) << std::endl;
                 std::cout << " last message printed: min k2 " << std::endl;
        	}
        	//xa[0] = q2x[1];
              //q2 = exp(xa[0]);
        */
        //std::cout << "  k2 out of range " << exp(xa[0]) << "  min " << exp(q2x[1]) << std::endl;
        return ;
    }
    if (xa[0] >= q2x[51]) {
        ++n1max;
        /*
          if (n1max < 10) {
             std::cout << " k2 out of range " << q2 << "  max " << exp(q2x[51]) << std::endl;
        } else if (n1max == 10) {
             std::cout << " k2 out of range " << q2 << " max " <<exp(q2x[51])  << std::endl;
             std::cout << " last message printed: max k2 " <<std::endl;
        }
        */
        //std::cout << " k2 out of range " << q2 << " max " <<exp(q2x[51])  << std::endl;
        //xa[0] = q2x[51] - .1;
        return ;
        //q2 = exp(xa[0]);
    }
    //std::cout << " after params in ccmfupf x: " << xa[1] << " limits " << xx[1] << " " << xx[51] <<  std::endl;
    //std::cout << " after params in ccmfupf kt2: " << xa[0] << " limits " << q2x[1] << " " << q2x[51] <<  std::endl;
    //std::cout << " after params in ccmfupf p: " << xa[2] << " limits " << px[1] << " " << px[51] <<  std::endl;
    /* check if interpolation or grid wanted */
    igrid = 0;
    if (igrid == 1) {
        if (xa[0] < q2x[1]) {
            std::cout << "  k2 out of range " << q2 <<  " min " << exp(q2x[1]) << " " << q2x[1] <<" "<< q2x[51] << std::endl;
            xa[0] = q2x[1];
        }
        if (xa[0] >= q2x[51]) {
            std::cout<<  "  k2 out of range " << q2 <<  " max " << exp(q2x[51]) << std::endl;
            xa[0] = q2x[51] - .1;
        }
        i = 0;
L20:
        ++i;
        if (xa[0] > a[na[0] - 1]) {
            /*       write(6,*) ' q2  ',xa(1),a(na(1)) */
            std::cout << " q2 not found " << q2 << std::endl;
            i = na[0];
        } else {
            if (xa[0] >= a[i - 1] && xa[0] < a[i]) {
            } else {
                if (i <= na[0]) {
                    goto L20;
                } else {
                    std::cout <<  " at 20: q2 not found " << q2 << std::endl;
                }
            }
        }
        j = 0;
L30:
        ++j;
        if (xa[1] >= a[na[0] + j - 1] && xa[1] < a[na[0] + j]) {
        } else {
            if (j <= na[1]) {
                goto L30;
            } else {
                std::cout << " at 30: x not found " << x << std::endl;
            }
        }
        k = 0;
L40:
        ++k;
        if (xa[2] >= a[na[0] + na[1] + na[2] - 1]) {
            k = na[2];
        } else {
            if (xa[2] >= a[na[0] + na[1] + k - 1] && xa[2] < a[na[0] + na[1]
                    + k]) {
            } else {
                if (k <= na[2]) {
                    goto L40;
                } else {
                    std::cout << " at 40: p not found " << p << std::endl;
                }
            }
        }
        glu = f_grid0[i + (j + k * 51) * 51 - 2653] / q2;
        up   = f_grid2[i + (j + k * 51) * 51 - 2653] / q2;
        ubar = f_grid2m[i + (j + k * 51) * 51 - 2653] / q2;
        dn   = f_grid1[i + (j + k * 51) * 51 - 2653] / q2;
        dbar = f_grid1m[i + (j + k * 51) * 51 - 2653] / q2;
        strange = f_grid3[i + (j + k * 51) * 51 - 2653] / q2;
        sbar  = f_grid3m[i + (j + k * 51) * 51 - 2653] / q2;
        charm = f_grid4[i + (j + k * 51) * 51 - 2653] / q2;
        cbar  = f_grid4m[i + (j + k * 51) * 51 - 2653] / q2;
        bottom = f_grid5[i + (j + k * 51) * 51 - 2653] / q2;
        bbar  = f_grid5m[i + (j + k * 51) * 51 - 2653] / q2;
        top   = f_grid6[i + (j + k * 51) * 51 - 2653] / q2;
        tbar  = f_grid6m[i + (j + k * 51) * 51 - 2653] / q2;
        phot  = f_grid7[i + (j + k * 51) * 51 - 2653] / q2;
        // std::cout << " allFlavuPDFn i j k "<< i << " " << j << " " << k << " " << glu << " " << q2 << " " << x << " " << p << std::endl;

    } else {
        int c3 = 3;
        // std::cout << " xa values  " << exp(xa[0]) << " " << exp(xa[1]) << " " << exp(xa[2]) << std::endl;
        glu   = Cdhfint(c3, xa, na, a, f_grid0) / q2;
        up    = Cdhfint(c3, xa, na, a, f_grid2) / q2;
        ubar  = Cdhfint(c3, xa, na, a, f_grid2m) / q2;
        uval  = up - ubar ;
        dn    = Cdhfint(c3, xa, na, a, f_grid1) / q2;
        dbar  = Cdhfint(c3, xa, na, a, f_grid1m) / q2;
        dval  = dn - dbar ;

        strange= Cdhfint(c3, xa, na, a, f_grid3) / q2;
        sbar   = Cdhfint(c3, xa, na, a, f_grid3m) / q2;
        charm  = Cdhfint(c3, xa, na, a, f_grid4) / q2;
        cbar   = Cdhfint(c3, xa, na, a, f_grid4m) / q2;
        bottom  = Cdhfint(c3, xa, na, a, f_grid5) / q2;
        bbar    = Cdhfint(c3, xa, na, a, f_grid5m) / q2;
        top     = Cdhfint(c3, xa, na, a, f_grid6) / q2;
        tbar    = Cdhfint(c3, xa, na, a, f_grid6m) / q2;
        phot    = Cdhfint(c3, xa, na, a, f_grid7) / q2;

    }


    //std::cout << " allFlavuPDFn: iset = " << iset << " gluon " << glu<< std::endl;

    //std::cout << " allFlavuPDFn: charm " << charm << " " << cbar << " " << q2 << std::endl;
    //std::cout << " allFlavuPDFn: gluon = " << glu << " " << q2 << std::endl;
    if(up == 0 || ubar == 0) {
        //std::cout << " allFlavuPDFn: up= " << up << " ubar= "<< ubar << " " << x << " " << q2 << " " << p << std::endl;
        //std::cout << " allFlavuPDFn: dn= " << dn << " dbar= "<< dbar << " " << x << " " << q2 << " " << p << std::endl;
    }
    //std::cout << " allFlavuPDFn x,q2,p,xpq " << x<< " " << q2 << " " << p << " "<< glu<< std::endl;
    if (glu > 1e6) {
        std::cout << " allFlavuPDFn x,q2,p,xpq" << x<< " " << q2 << " " << p << " "<< glu<< std::endl;
        std::cout << " i,j,k " << i << " " << j << " " << k << std::endl;
    }
    return ;
}

