//#define DEBUG

#include "tmdlib/TMDlib.h"
// LHAPDF parton distribution function (C++ wrapper)
//#include "LHAPDF/FortranWrappers.h"
#include "LHAPDF/LHAPDF.h"


#include <iostream>
#include <sstream>
#include <string>
#include <fstream>
using namespace LHAPDF;
using namespace TMDlib;


/* Common Block Declarations */

int iset;
extern std::string pdfpath;

bool first_sbrs = true;

void TMD::sbrsPDF(int irep, int imode, double x, double kt, double p, double& uval, double& dval, double& sea, double& charm)
{

    //Declare variables here

    const int fMem=200;   //Maximum number of replicas
    const int fNX=100;    //Maximum number of x-grid points
    const int fNQ2=2;     //Maximum number of Q2-grid points
    const int fNkt=50;    //Maximum number of kt-grid points
    const int fNFL=4;     //Maximum number of TMD PDFs

    const int m=4;
    const int n=1;
    const int o=4;

    const double pi = 3.14159;

    const double xmindata=0.06;
    const double xmaxdata=0.41;
    const double ktmaxdata=1.0;

    static const PDF* pdf;

    static int nrep, nxgrid, nktgrid, nQ2grid;
    int idum;

    int krep;
    int minx, maxx, midx, ix;
    int minQ, maxQ, midQ, iQ2;
    int mink, maxk, midk, ik;

    int ix1a[m], ix2a[n], ix3a[o];

    static double xminim, xmaxim, ktminim, ktmaxim, Q2minim, Q2maxim;
    static double xgrid[fNX], ktgrid[fNkt], Q2grid[fNQ2];

    static double xTMD[fMem][fNX][fNkt][fNQ2][fNFL];
    static double Nptdv[fMem], Nptuv[fMem], Nptsea[fMem];
    static double alpha[fMem], sigma[fMem];
    static double Nktfv[fMem], Nktunfv[fMem], NktKs[fMem], NktKu[fMem];
    static double beta[fMem], Gamma[fMem], delta[fMem];
    static double dln[fMem], nPKm[fMem], nDKm[fMem];

    double xpdf[fNFL];

    double TMuval[fMem], TMdval[fMem], TMsea[fMem];

    double y, dy, x1, x2, x3;
    static double x1a[fNX], x2a[fNkt], x3a[fNQ2];
    double ya[m][n][o];

    double p2=pow(p,2);
    const double xcha=0.1;

    x1=x;
    x2=p2;
    x3=kt;


    std::string filename;

    //Define path to grid
    if (pdfpath!="")
    {
        //std::cout << "TMDlib::sbrsPDF TMDlibPATH = " <<  pdfpath << std::endl;
    }
    else
    {
        std::cout << " NO TMDlibPATH  set " <<std::endl;
        return ;
    }

    if (iset >= 300001 && iset <= 300003 )
    {
        //if (imode==0||imode==2)
        if ( iset == 300001||iset == 300003 )
        {
            filename=pdfpath+TMD_Dir+"SBRS-2013-TMDPDFs-par.dat";
        }

        // else if (imode==1)
        else if ( iset == 300002 )
        {
            filename=pdfpath+TMD_Dir+"SBRS-2013-TMDPDFs.dat";
        }
        else
        {
            //std::cout << "  Invalid value for imode " << std::endl;
            //std::cout << "  imode = 0: analytc form " << std::endl;
            //std::cout << "  imode = 1: interpolation on the grid " << std::endl;
            //std::cout << "  imode = 2: analytic form, bT space " << std::endl;
            std::cout << "  Invalid value for imode " << std::endl;
            std::cout << "  iset = 300001: analytc form " << std::endl;
            std::cout << "  iset = 300002: interpolation on the grid " << std::endl;
            std::cout << "  iset = 300003: analytic form, bT space " << std::endl;

        }
    }
    else
    {
        std::cout << "  TMDlib:sbrsPDF set not found; iset = " << iset << std::endl;
        return ;
    }

    //Read grid file
    if (first_sbrs)
    {
        // std::cout << "sbrsPDF: read grid from " << filename << std::endl;

        //if(imode==0||imode==2)
        if(iset==300001||iset==300003)
        {
#ifdef DEBUG
            std::cout << "  Initialization of TMD PDF analytic form " << std::endl;
#endif
            //Read best-fit parameters from file
            ifstream myfile;
            std::string textdum;
            myfile.open(filename.c_str());
            if(!myfile) {std::cout << " The file cannot be opened " << std::endl; exit(1);}
            else
            {
                //Read header and print to screen
                for (;;)
                {
                    getline(myfile,textdum);
                    if (textdum.find("MinMax:") != string::npos) break;
#ifdef DEBUG
                    std::cout << textdum << std::endl;
#endif
                }

                //Read min and max values
                myfile >> nrep >> textdum >> idum;
                myfile >> xminim >> textdum >> xmaxim >> textdum >> ktminim >> textdum >> ktmaxim >> textdum >> Q2minim >> textdum >> Q2maxim;

                for (;;)
                {
                    getline(myfile,textdum);
                    if (textdum.find("Some") != string::npos) break;
#ifdef DEBUG
                    std::cout << textdum << std::endl;
#endif
                }
                myfile >> nrep;
                for(int krepwrap=0; krepwrap<nrep; krepwrap++)
                {
                    myfile >> Nptdv[krepwrap]
                           >> Nptuv[krepwrap]
                           >> Nptsea[krepwrap]
                           >> alpha[krepwrap] >> sigma[krepwrap]
                           >> Nktfv[krepwrap] >> Nktunfv[krepwrap]
                           >> NktKs[krepwrap] >> NktKu[krepwrap]
                           >> beta[krepwrap] >> Gamma[krepwrap] >> delta[krepwrap]
                           >> dln[krepwrap] >> nPKm[krepwrap] >> nDKm[krepwrap];
                }
            }
            myfile.close();

            //Initialize MSTW 68% CL fit
#ifdef DEBUG
            std::cout << "  Initialize collinear PDFs " << std::endl;
#endif
            const int SUBSET = 0;
            const std::string NAME = "MSTW2008lo68cl";
            //	  LHAPDF::initPDFSet(NAME, LHAPDF::LHGRID, SUBSET);
            //      LHAPDF::PDF* pdf = LHAPDF::mkPDF(NAME,SUBSET);
            pdf = mkPDF(NAME,SUBSET);
        }

        //else if(imode==1)
        else if(iset==300002)
        {
            // std::cout << "  Initialization of TMD PDF grid for interpolation " << std::endl;

            ifstream myfile;
            std::string textdum;
            myfile.open(filename.c_str());
            if(!myfile) {std::cout << " The file cannot be opened " << std::endl; exit(1);}
            else
            {
                //Read header and print to screen
                for (;;)
                {
                    getline(myfile,textdum);
                    if (textdum.find("MinMax:") != string::npos) break;
#ifdef DEBUG
                    std::cout << textdum << std::endl;
#endif
                }

                //Read min and max values
                myfile >> nrep >> textdum >> idum;
                myfile >> xminim >> textdum >> xmaxim >> textdum >> ktminim >> textdum >> ktmaxim >> textdum >> Q2minim >> textdum >> Q2maxim;

                for (;;)
                {
                    getline(myfile,textdum);
                    if (textdum.find("Some") != string::npos) break;
#ifdef DEBUG
                    std::cout << textdum << std::endl;
#endif
                }

                myfile >> nrep;

                myfile >> nxgrid;

                for(int i=0; i<nxgrid; i++)
                {
                    myfile >> xgrid[i];
                }

                myfile >> nktgrid;

                for(int i=0; i<nktgrid; i++)
                {
                    myfile >> ktgrid[i];
                }

                myfile >> nQ2grid;

                for(int i=0; i<nQ2grid; i++)
                {
                    myfile >> Q2grid[i];
                }

                myfile >> nrep;

                for(int ir=0; ir<nrep; ir++)
                {
                    for(int a=0; a<nxgrid; a++)
                    {
                        for (int b=0; b<nktgrid; b++)
                        {
                            for(int c=0; c<nQ2grid; c++)
                            {
                                double PDF1, PDF2, PDF3, PDF4;
                                myfile >> xTMD[ir][a][b][c][0]
                                       >> xTMD[ir][a][b][c][1]
                                       >> xTMD[ir][a][b][c][2]
                                       >> xTMD[ir][a][b][c][3];
                            }
                        }
                    }
                }
                myfile >> textdum;
            }

            myfile.close();
        }
        first_sbrs = false;
        // std::cout << "  Parton densities read from file " << std::endl;
    }

    //Check interpolation range
    if (x<xminim || x>xmaxim)
    {
        std::cout << "  Parton interpolation out of range: "
                  << " xmin = " << xminim << " xmax = " << xmaxim
                  << " x = " << x << std::endl;
        // exit(1);
    }
    if (kt<ktminim || kt > ktmaxim)
    {
        std::cout << "  Parton interpolation out of range: "
                  << " ktmin = " << ktminim << " GeV "
                  << " ktmax = " << ktmaxim << " GeV "
                  << " kt = " << kt << " GeV " << std::endl;
        //  exit(1);
    }
    if (p<sqrt(2.3)||p>sqrt(2.5))
    {
        std::cout << "  No evolution available: "
                  << " Qmin = " << sqrt(Q2minim) << " GeV^2 " << " p = " << p<<  std::endl;
        std::cout << " program stopped " << std::endl;
        //  exit(1);
    }

    //Check extrapolation region and print warnings
    if (x<xmindata || x>xmaxdata)
    {
        std::cout << "  TMDlib SBRS: Warning: extrapolation made outside data region" << std::endl;
        std::cout << "  TMDlib SBRS: The output may not be reliable at x = " << x << std::endl;
        std::cout << "  TMDlib SBRS: xmin = " << xmindata << " xmax = " << xmaxdata << std::endl;
    }
    if (kt>ktmaxdata)
    {
        std::cout << "  TMDlib SBRS: Warning: extrapolation made outside data region" << std::endl;
        std::cout << "  TMDlib SBRS: The output may not be reliable at kt = " << kt << " GeV " << std::endl;
        std::cout << "  TMDlib SBRS: ktmax = " << ktmaxdata << std::endl;
    }

    if(irep<0||irep>nrep)
    {
        std::cout << "invalid value for irep = " << irep << std::endl;
        exit(1);
    }

    //Return TMD-PDFs by using their analytic form
    //if(imode==0)
    if (iset == 300001)
    {

        //Compute average transverse momenta
        for(int krepwrap=0; krepwrap<nrep; krepwrap++)
        {
            TMuval[krepwrap]= Nptuv[krepwrap]*( pow(1-x,alpha[krepwrap])*pow(x,sigma[krepwrap]) )/( pow(1-0.1,alpha[krepwrap])*pow(0.1,sigma[krepwrap]) );
            TMdval[krepwrap] = Nptdv[krepwrap]*( pow(1-x,alpha[krepwrap])*pow(x,sigma[krepwrap]) )/( pow(1-0.1,alpha[krepwrap])*pow(0.1,sigma[krepwrap]) );
            TMsea[krepwrap] = Nptsea[krepwrap]*( pow(1-x,alpha[krepwrap])*pow(x,sigma[krepwrap]) )/( pow(1-0.1,alpha[krepwrap])*pow(0.1,sigma[krepwrap]) );
        }
        double Qmeanfit;
        Qmeanfit = p;

        uval = ( pdf->xfxQ(2,x,Qmeanfit) - pdf->xfxQ(-2,x,Qmeanfit) )*( (1/(pi*TMuval[irep]))*exp( -pow(kt,2)/TMuval[irep] ) );

        dval = ( pdf->xfxQ(1,x,Qmeanfit) - pdf->xfxQ(-1,x,Qmeanfit) )*( (1/(pi*TMdval[irep]))*exp( -pow(kt,2)/TMdval[irep] ) );

        sea = pdf->xfxQ(3,x,Qmeanfit)*( (1/(pi*TMsea[irep]))*exp( -pow(kt,2)/TMsea[irep] ) );

        charm = pdf->xfxQ(4,x,Qmeanfit)*( (1/(pi*TMsea[irep]))*exp( -pow(kt,2)/TMsea[irep] ) );

        // std::cout << " SBRS " << uval << " " << dval << " " << sea << std::endl;

    }

    //Return TMD-PDFs by interpolating on the grid file
    //else if(imode==1)
    else if (iset==300002)
    {
        //Find nearest points for grid interpolation
        minx=0;
        maxx=nxgrid;
        do
        {
            midx=(minx+maxx)/2;
            if(x<xgrid[midx])
            {maxx=midx;}
            else
            {minx=midx;}
        }
        while ((maxx-minx)>1);
        ix=minx;

        minQ=0;
        maxQ=nQ2grid;
        do
        {
            midQ=(minQ+maxQ)/2;
            if(p2<Q2grid[midQ])
            {maxQ=midQ;}
            else
            {minQ=midQ;}
        }
        while ((maxQ-minQ)>1);
        iQ2=minQ;

        mink=0;
        maxk=nktgrid;
        do
        {
            midk=(mink+maxk)/2;
            if(kt<ktgrid[midk])
            {maxk=midk;}
            else
            {mink=midk;}
        }
        while ((maxk-mink)>1);
        ik=mink;

        for(int i=0; i<m; i++)
        {
            if(ix>=m/2&&ix<=(nxgrid-m/2)) {ix1a[i]=ix-m/2+i;}
            if(ix<m/2) {ix1a[i]=i;}
            if(ix>(nxgrid-m/2)) {ix1a[i]=(nxgrid-m)+i;}
            // std::cout << " SBRS nxgrid " << nxgrid << " " << m << std::endl;
            if(ix1a[i]<0||ix1a[i]>=nxgrid)
            {
                std::cout << "Error in grids!" << std::endl;
                std::cout << "i, ix1a[i] = " << i << "  " << ix1a[i] << std::endl;
                exit(1);
            }
        }

        for(int j=0; j<n; j++)
        {
            if(iQ2>=n/2&&iQ2<=(nQ2grid-n/2)) {ix2a[j]=iQ2-n/2+j;}
            if(iQ2<n/2) {ix2a[j]=j;}
            if(iQ2>(nQ2grid-n/2)) {ix2a[j]=(nQ2grid-n)+j;}

            if(ix2a[j]<0||ix2a[j]>=nQ2grid)
            {
                std::cout << "Error in grids!" << std::endl;
                std::cout << "j, ix2a[j] = " << j << "  " << ix2a[j] << std::endl;
                exit(1);
            }
        }

        for(int l=0; l<o; l++)
        {
            if(ik>=o/2&&ik<=(nktgrid-o/2)) {ix3a[l]=ik-o/2+l;}
            if(ik<o/2) {ix3a[l]=l;}
            if(ik>(nktgrid-o/2)) {ix3a[l]=(nktgrid-o)+l;}

            if(ix3a[l]<0||ix3a[l]>=nktgrid)
            {
                std::cout << "Error in grids!" << std::endl;
                std::cout << "l, ix3a[l] = " << l << "  " << ix3a[l] << std::endl;
                exit(1);
            }
        }


        for(int jpdf=0; jpdf<fNFL; jpdf++)
        {
            for(int i=0; i<m; i++)
            {
                x1a[i]=xgrid[ix1a[i]];

                for(int j=0; j<n; j++)
                {
                    x2a[j]=Q2grid[ix2a[j]];

                    for(int l=0; l<o; l++)
                    {
                        x3a[l]=ktgrid[ix3a[l]];
                        ya[i][j][l]=xTMD[irep][ix1a[i]][ix3a[l]][ix2a[j]][jpdf];
                    }
                }
            }
            polin3(x1a,x2a,x3a,ya,m,n,o,x1,x2,x3,y,dy);
            xpdf[jpdf]=y;
        }

        uval = xpdf[0];
        dval = xpdf[1];
        sea = xpdf[2];
        charm = xpdf[3];
    }

    //Return the FT in bT space for TMD-PDFs using the analytic result
    //if(imode==2)
    if(iset == 300003)
    {

        //Compute average transverse momenta
        for(int krepwrap=0; krepwrap<nrep; krepwrap++)
        {
            TMuval[krepwrap]= Nptuv[krepwrap]*( std::pow(1-x,alpha[krepwrap])*std::pow(x,sigma[krepwrap]) )/( std::pow(1-0.1,alpha[krepwrap])*std::pow(0.1,sigma[krepwrap]) );
            TMdval[krepwrap] = Nptdv[krepwrap]*( std::pow(1-x,alpha[krepwrap])*std::pow(x,sigma[krepwrap]) )/( std::pow(1-0.1,alpha[krepwrap])*std::pow(0.1,sigma[krepwrap]) );
            TMsea[krepwrap] = Nptsea[krepwrap]*( std::pow(1-x,alpha[krepwrap])*std::pow(x,sigma[krepwrap]) )/( std::pow(1-0.1,alpha[krepwrap])*std::pow(0.1,sigma[krepwrap]) );
        }
        double Qmeanfit;
        Qmeanfit = p;

        uval = ( pdf->xfxQ(2,x,Qmeanfit) - pdf->xfxQ(-2,x,Qmeanfit) )*std::exp(-std::pow(kt,2)*TMuval[irep]/4.0);

        dval = ( pdf->xfxQ(1,x,Qmeanfit) - pdf->xfxQ(-1,x,Qmeanfit) )*std::exp(-std::pow(kt,2)*TMdval[irep]/4.0 );

        sea = pdf->xfxQ(3,x,Qmeanfit)*std::exp(-std::pow(kt,2)*TMsea[irep]/4.0 );

        charm = pdf->xfxQ(4,x,Qmeanfit)*std::exp(-std::pow(kt,2)*TMsea[irep]/4.0 );

    }

    return ;

}
